import os

import pytest

from import_dbf.util import (
    addslashes,
    dbf2sql,
    exp_add_spacing,
    exp_parse_str,
    xls2dict,
    exp_split_poz_str,
)


def test_util_addslashes():
    assert addslashes(None) is None
    assert addslashes(1) == 1
    assert addslashes("foo'") == "foo''"


def test_util_import_dbf():
    dbf2sql(os.path.join(os.path.dirname(__file__), "test.dbf"))
    open(os.path.join(os.path.dirname(__file__), "test.dbf") + ".sql")


@pytest.mark.parametrize(
    "input, output",
    [
        (
            "#204$ #a$ Zachowania zdrowotne dzieci w wieku szkolnym - badania wstępne #b$ #c$ #d$ #e$ s.184-191 "
            "#f$ ryc., tab., bibliogr., streszcz.",
            {
                "id": 204,
                "a": "Zachowania zdrowotne dzieci w wieku szkolnym - badania wstępne",
                "b": "",
                "c": "",
                "d": "",
                "e": "s.184-191",
                "f": "ryc., tab., bibliogr., streszcz.",
            },
        ),
        (
            "#102$ #a$ Autologous hematopoietic stem cell transplant for progressive"
            " diffuse systemic sclerosis: procedural "
            "success and clinical outcome in 5-year follow-up #b$ #c$ #d$",
            {
                "id": 102,
                "a": "Autologous hematopoietic stem cell transplant for progressive diffuse systemic sclerosis: "
                "procedural success and clinical outcome in 5-year follow-up",
                "b": "",
                "c": "",
                "d": "",
            },
        ),
        (
            "#150$ #a$ Badania nad wpływem selenu na integralność chromatyny plemnikowej w przypadkach mężczyzn z "
            "upośledzoną płodnością #b$ #c$ #d$",
            {
                "id": 150,
                "a": "Badania nad wpływem selenu na integralność chromatyny plemnikowej w przypadkach mężczyzn "
                "z upośledzoną płodnością",
                "b": "",
                "c": "",
                "d": "",
            },
        ),
        (
            "#150$ #a$Neurologia #c$Howard L. Weiner, Lawrence P. Levitt	",
            {"id": 150, "a": "Neurologia", "c": "Howard L. Weiner, Lawrence P. Levitt"},
        ),
        (
            "#103$#a$ Abstracts of the 5-th World Congress on Heart Failure - Mechanisms and Managment. "
            "Washinfton, USA, 11-14 May, 1997",
            {
                "id": 103,
                "a": "Abstracts of the 5-th World Congress on Heart Failure - Mechanisms and Managment. "
                "Washinfton, USA, 11-14 May, 1997",
            },
        ),
    ],
)
def test_util_exp_parse_str(input, output):
    assert exp_parse_str(input) == output


@pytest.mark.parametrize(
    "input,output",
    [
        (
            "#153$ #a$ 162 s. #b$ bibliogr. #c$\r\n#154$ #a$|0000004804#b$ #c$\r\n"
            "nr 154 #d$ #e$\r\n#155$ #a$ ISBN 978-83-66066-86-1#988$ #a$|0000\r\n"
            "006505#b$|0000006506#991$ #a$ 10.34616/23.19.146\r\n#995$ #a$ http\r\n"
            "s://www.bibliotekacyfrowa.pl/publication/108240\r\n               ",
            [
                "#153$ #a$ 162 s. #b$ bibliogr. #c$",
                "#154$ #a$|0000004804#b$ #c$nr 154 #d$ #e$",
                "#155$ #a$ ISBN 978-83-66066-86-1",
                "#988$ #a$|0000006505#b$|0000006506",
                "#991$ #a$ 10.34616/23.19.146",
                "#995$ #a$ https://www.bibliotekacyfrowa.pl/publication/108240               ",
            ],
        )
    ],
)
def test_util_exp_split_poz_str(input, output):
    assert output == list(exp_split_poz_str(input))


def test_util_exp_parse_str_raises():
    bad = (
        "#105$ Electrophysiological estimation of the peripheral nerves conduction "
        "parameters and the autonomic nervous system function in the course"
        " of amyotrophic lateral sclerosis #b$"
    )
    with pytest.raises(ValueError):
        exp_parse_str(bad)


def test_util_exp_add_spacing():
    assert exp_add_spacing(" te  st (fa. ) ") == " te st (fa.) "


def test_xls2dict():
    ret = list(xls2dict(os.path.join(os.path.dirname(__file__), "test_chf.xlsx")))
    assert ret[0]["skrot"] == "PAA"



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-14 16:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('import_dyscyplin', '0005_auto_20180414_1801'),
    ]

    operations = [
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='info',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='stan',
            field=models.CharField(choices=[('nowy', 'nowy'), ('błędny', 'błędny'), ('zintegrowany', 'zintegrowany')], default='nowy', max_length=50),
        ),
    ]



import os

import pytest
from django.core.files.base import ContentFile
from django.db import transaction

from bpp.models import Autor_Dyscyplina, Dyscyplina_Naukowa
from import_dyscyplin.models import (
    Import_Dyscyplin,
    Import_Dyscyplin_Row,
    Kolumna,
    guess_rodzaj,
)


def test_guess_rodzaj():
    assert guess_rodzaj("nazwisko") == Kolumna.RODZAJ.NAZWISKO
    assert guess_rodzaj("kopara") == Kolumna.RODZAJ.POMIJAJ
    assert guess_rodzaj("lp") == Kolumna.RODZAJ.POMIJAJ


def test_Import_Dyscyplin_post_delete_handler(
    test1_xlsx, normal_django_user, transactional_db
):
    path = None
    with transaction.atomic():
        i = Import_Dyscyplin.objects.create(
            owner=normal_django_user,
        )

        i.plik.save("test1.xls", ContentFile(open(test1_xlsx, "rb").read()))
        path = i.plik.path
        i.delete()

    assert not os.path.exists(path)


@pytest.fixture
def testowe_dyscypliny():
    Dyscyplina_Naukowa.objects.create(nazwa="Testowa", kod="3.2", widoczna=False)

    Dyscyplina_Naukowa.objects.create(nazwa="Jakaś", kod="3.2.1", widoczna=False)


@pytest.fixture
def id_row_1(import_dyscyplin, autor_jan_nowak):
    return Import_Dyscyplin_Row.objects.create(
        parent=import_dyscyplin,
        row_no=1,
        original={},
        dyscyplina="Testowa",
        kod_dyscypliny="3.2",
        subdyscyplina="Jakaś",
        kod_subdyscypliny="3.2.1",
        autor=autor_jan_nowak,
    )


@pytest.fixture
def id_row_2(import_dyscyplin, autor_jan_nowak):
    return Import_Dyscyplin_Row.objects.create(
        parent=import_dyscyplin,
        row_no=1,
        original={},
        dyscyplina="Testowa",
        kod_dyscypliny="3.2",
        subdyscyplina=None,
        kod_subdyscypliny=None,
        autor=autor_jan_nowak,
    )


def test_Import_Dyscyplin_integruj_dyscypliny_pusta_baza(import_dyscyplin, id_row_1):
    import_dyscyplin.integruj_dyscypliny()

    assert Dyscyplina_Naukowa.objects.all().count() == 2
    Dyscyplina_Naukowa.objects.get(nazwa="Testowa")

    for elem in Dyscyplina_Naukowa.objects.all():
        assert elem.widoczna


def test_Import_Dyscyplin_integruj_dyscypliny_ta_sama_nazwa_inny_kod(
    import_dyscyplin, id_row_1
):
    Dyscyplina_Naukowa.objects.create(nazwa="Testowa", kod="0.0")

    import_dyscyplin.integruj_dyscypliny()

    assert Dyscyplina_Naukowa.objects.all().count() == 1

    id_row_1.refresh_from_db()
    assert id_row_1.stan == Import_Dyscyplin_Row.STAN.BLEDNY


def test_Import_Dyscyplin_integruj_dyscypliny_ta_sama_nazwa_inny_kod_sub(
    import_dyscyplin, id_row_1
):
    Dyscyplina_Naukowa.objects.create(nazwa="Jakaś", kod="5.3")

    import_dyscyplin.integruj_dyscypliny()

    assert Dyscyplina_Naukowa.objects.all().count() == 2

    id_row_1.refresh_from_db()
    assert id_row_1.stan == Import_Dyscyplin_Row.STAN.BLEDNY


def test_Import_Dyscyplin_integruj_dyscypliny_ukryj_nieuzywane_brak_dyscyplin(
    import_dyscyplin, id_row_1
):
    import_dyscyplin.integruj_dyscypliny()
    Autor_Dyscyplina.objects.ukryj_nieuzywane()
    assert Dyscyplina_Naukowa.objects.all().count() == 2
    for elem in Dyscyplina_Naukowa.objects.all():
        assert not elem.widoczna


def test_Import_Dyscyplin_integruj_dyscypliny_ukryj_nieuzywane_uzywana_nadrzedna(
    import_dyscyplin, id_row_2, autor_jan_nowak, testowe_dyscypliny
):
    assert Autor_Dyscyplina.objects.count() == 0

    import_dyscyplin.integruj_dyscypliny()
    import_dyscyplin._integruj_wiersze()

    assert Autor_Dyscyplina.objects.count() == 1
    assert Dyscyplina_Naukowa.objects.get(nazwa="Testowa").widoczna
    assert not Dyscyplina_Naukowa.objects.get(nazwa="Jakaś").widoczna


def test_Import_Dyscyplin_integruj_dyscypliny_ukryj_nieuzywane_uzywana_podrzedna(
    import_dyscyplin, id_row_1, testowe_dyscypliny
):
    assert Autor_Dyscyplina.objects.count() == 0

    import_dyscyplin.integruj_dyscypliny()
    import_dyscyplin._integruj_wiersze()
    assert Autor_Dyscyplina.objects.count() == 1

    assert Dyscyplina_Naukowa.objects.get(nazwa="Testowa").widoczna
    assert Dyscyplina_Naukowa.objects.get(nazwa="Jakaś").widoczna


def test_Import_Dyscyplin_sprawdz_czy_poprawne(
    import_dyscyplin, autor_jan_nowak, id_row_1
):
    id_row_2 = Import_Dyscyplin_Row.objects.create(
        parent=import_dyscyplin,
        row_no=1,
        original={},
        dyscyplina="Testowa",
        kod_dyscypliny="3.2",
        subdyscyplina="Jakaś",
        kod_subdyscypliny="3.2.1",
        autor=autor_jan_nowak,
    )

    import_dyscyplin.integruj_dyscypliny()

    assert id_row_1 in list(import_dyscyplin.poprawne_wiersze_do_integracji())

    import_dyscyplin.sprawdz_czy_poprawne()

    for elem in id_row_1, id_row_2:
        elem.refresh_from_db()
        assert elem.stan == Import_Dyscyplin_Row.STAN.BLEDNY


def test_Import_Dyscyplin_integruj_dyscypliny_zmiana_dyscypliny(
    autor_jan_nowak, rok, import_dyscyplin, id_row_1, testowe_dyscypliny
):
    # Zróbmy tak, że autor ma przypisanie za dany rok dla dyscypliny
    # innej, niż w wierszu importu. W wierszu id_row_1 idzie 'Testowa'
    # jako dyscyplina...
    ad = Autor_Dyscyplina.objects.create(
        autor=autor_jan_nowak,
        rok=rok,
        dyscyplina_naukowa=Dyscyplina_Naukowa.objects.get(nazwa="Jakaś"),
    )

    import_dyscyplin.integruj_dyscypliny()
    import_dyscyplin._integruj_wiersze()

    ad.refresh_from_db()
    assert ad.dyscyplina_naukowa.nazwa == "Testowa"


def test_Import_Dyscyplin_Row_serialize_dict():
    x = Import_Dyscyplin_Row(nazwisko="foo", imiona="bar", original={})
    assert x.serialize_dict()["nazwisko"] == "foo"



# -*- encoding: utf-8 -*-
import pytest
from django.urls.base import reverse

from django.utils.http import urlencode

from bpp.models.autor import Autor


def test_search_field(autor_jan_kowalski):
    assert Autor.objects.all()[0].search is not None


def test_tokenizer_minus(wydawnictwo_zwarte, client):
    wydawnictwo_zwarte.tytul_oryginalny = "ewaluacja 2017-2020"
    wydawnictwo_zwarte.save()

    base_url = reverse("bpp:navigation-autocomplete")
    url = base_url + "?" + urlencode({"q": "2017-2020"})
    res = client.get(url)
    assert b"ewaluacja" in res.content

    url = base_url + "?" + urlencode({"q": "-2020"})
    res = client.get(url)
    assert b"ewaluacja" in res.content


def test_fulltext_search_mixin(autor_jan_kowalski):
    res = Autor.objects.fulltext_filter("kowalski jan")
    assert autor_jan_kowalski in res


@pytest.mark.django_db
@pytest.mark.parametrize(
    "s",
    [
        "śmierć",
        "śmierć",
        "pas ternak'",
        "paste rnak''",
        "past ernak\\",
        "pastern ak\\'",
        "past ernak'",
        "past ernak &",
        "pa sternak (",
        "!paster nak",
        "paste rnak)",
        "&",
        "& &",
        "()",
        "!!",
        "!",
        ")()()(\\\\!@!!@@!#!@",
        "   ()(*(*$(*#  oiad  9*(*903498985398)()(||| aosid  p p    ",
    ],
)
def test_global_nav_search(client, s):
    url = reverse("bpp:navigation-autocomplete")
    url += "?" + urlencode({"q": s})
    res = client.get(url)
    assert res.status_code == 200



from django.urls import reverse


def test_ListaImportowView_link(admin_app):
    page = admin_app.get(reverse("import_list_if:index"))
    page = page.click("pobierz plik wzorcowy")
    assert page.status_code == 200


def test_NowyImportView_link(admin_app):
    page = admin_app.get(reverse("import_list_if:new"))
    page = page.click("pobierz plik wzorcowy")
    assert page.status_code == 200



import pytest
from django.urls import reverse
from model_bakery import baker
from selenium.webdriver.support.wait import WebDriverWait

from pbn_api.admin import OswiadczeniaInstytucjiAdmin
from pbn_api.client import PBN_DELETE_PUBLICATION_STATEMENT
from pbn_api.models import OswiadczenieInstytucji, SentData
from pbn_api.tests.utils import middleware

from django.contrib.contenttypes.models import ContentType
from django.contrib.messages import get_messages

from django_bpp.selenium_util import LONG_WAIT_TIME, wait_for_page_load


def test_SentDataAdmin_list_filter_works(admin_client):
    url = reverse("admin:pbn_api_sentdata_changelist")
    res = admin_client.get(url + "?q=123")
    assert res.status_code == 200


@pytest.mark.django_db
def test_OswiadczenieInstytucji_delete_model(pbn_uczelnia, pbn_client, rf):
    oi = baker.make(OswiadczenieInstytucji)
    req = rf.get("/")

    pbn_client.transport.return_values[
        PBN_DELETE_PUBLICATION_STATEMENT.format(publicationId=oi.publicationId_id)
    ] = {"1": "2"}

    with middleware(req):
        OswiadczeniaInstytucjiAdmin(OswiadczenieInstytucji, None).delete_model(
            req, oi, pbn_client=pbn_client
        )

    try:
        OswiadczenieInstytucji.objects.get(pk=oi.pk)
        msg = list(get_messages(req))
        if msg:
            raise Exception(str(msg))
        raise Exception("Nie został skasowany")
    except OswiadczenieInstytucji.DoesNotExist:
        assert True  # good


def test_pbn_api_admin_SentDataAdmin_wyslij_ponownie(
    wydawnictwo_zwarte, admin_browser, asgi_live_server
):

    s = SentData.objects.create(
        object_id=wydawnictwo_zwarte.pk,
        content_type=ContentType.objects.get_for_model(wydawnictwo_zwarte),
        data_sent={"foo": "bar"},
    )

    with wait_for_page_load(admin_browser):
        admin_browser.visit(
            asgi_live_server.url + f"/admin/pbn_api/sentdata/{s.pk}/change"
        )

    elem = admin_browser.find_by_id("wyslij-ponownie")

    with wait_for_page_load(admin_browser):
        elem.click()

    WebDriverWait(admin_browser, LONG_WAIT_TIME).until(
        lambda *args, **kw: "nie będzie eksportowany" in admin_browser.html
    )
    assert True



# Generated by Django 3.0.14 on 2021-05-19 00:37

from django.db import migrations

import bpp.models.dyscyplina_naukowa
import bpp.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0266_auto_20210519_0236"),
    ]

    operations = [
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_liczbe_cytowan_na_stronie_autora",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="always",
                help_text="Liczba cytowań będzie wyświetlana, gdy większa od 0",
                max_length=50,
                verbose_name="Pokazuj liczbę cytowań na podstronie autora",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_liczbe_cytowan_w_rankingu",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="always",
                max_length=50,
                verbose_name="Pokazuj liczbę cytowań w rankingu",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_praca_recenzowana",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="always",
                max_length=50,
                verbose_name='Pokazuj opcję "Praca recenzowana"',
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_ranking_autorow",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="always",
                max_length=50,
                verbose_name="Pokazuj ranking autorów",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_raport_autorow",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="always",
                max_length=50,
                verbose_name="Pokazuj raport autorów",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_raport_dla_komisji_centralnej",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="always",
                max_length=50,
                verbose_name="Pokazuj raport dla Komisji Centralnej",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_raport_jednostek",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="logged-in",
                max_length=50,
                verbose_name="Pokazuj raport jednostek",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_raport_slotow_autor",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="logged-in",
                max_length=50,
                verbose_name="Pokazuj raport slotów - autor",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_raport_slotow_uczelnia",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="logged-in",
                max_length=50,
                verbose_name="Pokazuj raport slotów - uczelnia",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_raport_slotow_zerowy",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="logged-in",
                max_length=50,
                verbose_name="Pokazuj raport slotów zerowy",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_raport_wydzialow",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="logged-in",
                max_length=50,
                verbose_name="Pokazuj raport wydziałów",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_status_korekty",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="always",
                max_length=50,
                verbose_name="Pokazuj status korekty na stronie rekordu",
            ),
        ),
        migrations.AlterField(
            model_name="uczelnia",
            name="pokazuj_tabele_slotow_na_stronie_rekordu",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("staff", "tylko dla zespołu"),
                    ("never", "nigdy"),
                ],
                default="logged-in",
                max_length=50,
                verbose_name="Pokazuj tabelę slotów na stronie rekordu",
            ),
        ),
    ]



import pytest
from model_bakery import baker

from rozbieznosci_if.models import IgnorujRozbieznoscIf
from rozbieznosci_if.views import RozbieznosciView

from bpp.models import Wydawnictwo_Ciagle, Zrodlo


@pytest.fixture
def wydawnictwo_z_rozbieznoscia(rok):
    zrodlo = baker.make(Zrodlo)
    zrodlo.punktacja_zrodla_set.create(rok=rok, impact_factor=50)

    wydawnictwo_ciagle = baker.make(
        Wydawnictwo_Ciagle, impact_factor=10, rok=rok, zrodlo=zrodlo
    )

    return wydawnictwo_ciagle


@pytest.fixture
def wydawnictwo_bez_rozbieznosci(rok):
    zrodlo = baker.make(Zrodlo)
    zrodlo.punktacja_zrodla_set.create(rok=rok, impact_factor=50)

    wydawnictwo_ciagle = baker.make(
        Wydawnictwo_Ciagle, rok=rok, impact_factor=50, zrodlo=zrodlo
    )

    return wydawnictwo_ciagle


@pytest.mark.django_db
def test_RozbieznosciView_get_queryset_tak(wydawnictwo_z_rozbieznoscia):
    res = RozbieznosciView().get_queryset()
    assert wydawnictwo_z_rozbieznoscia in res


@pytest.mark.django_db
def test_RozbieznosciView_get_queryset_ignorowane(wydawnictwo_z_rozbieznoscia):
    IgnorujRozbieznoscIf.objects.create(object=wydawnictwo_z_rozbieznoscia)
    res = RozbieznosciView().get_queryset()
    assert wydawnictwo_z_rozbieznoscia not in res


@pytest.mark.django_db
def test_RozbieznosciView_get_queryset_nie(wydawnictwo_bez_rozbieznosci):
    res = RozbieznosciView().get_queryset()
    assert wydawnictwo_bez_rozbieznosci not in res


@pytest.mark.django_db
def test_RozbieznosciView_dodaj_do_ignorowanych(
    wydawnictwo_z_rozbieznoscia, rf, admin_user
):
    req = rf.get("/", data={"_ignore": str(wydawnictwo_z_rozbieznoscia.pk)})
    req.user = admin_user

    rv = RozbieznosciView(kwargs={}, request=req)
    # rv.request = req

    rv.get(req)
    rv.get(req)
    rv.get(req)

    assert IgnorujRozbieznoscIf.objects.count() == 1


@pytest.mark.django_db
def test_RozbieznosciView_ustaw(wydawnictwo_z_rozbieznoscia, rf, admin_user):
    req = rf.get("/", data={"_set": str(wydawnictwo_z_rozbieznoscia.pk)})
    req.user = admin_user

    rv = RozbieznosciView(kwargs={}, request=req)
    # rv.request = req

    rv.get(req)

    wydawnictwo_z_rozbieznoscia.refresh_from_db()
    assert wydawnictwo_z_rozbieznoscia.impact_factor == 50



# Create your tests here.
from decimal import Decimal

import pytest

from ewaluacja2021 import const
from ewaluacja2021.tests.utils import curdir
from ewaluacja2021.util import (
    InputXLSX,
    chunker,
    float_or_string_or_int_or_none_to_decimal,
    normalize_xlsx_header_column_name,
)


def test_chunker():
    assert list(
        chunker(
            3,
            [
                [0, 1, 2, 3],
                [0, 1, 2, 3],
                [0, 1, 2, 3],
                [3, 4, 5, 6],
                [3, 4, 5, 6],
                [3, 4, 5],
                [7, 8, 9],
            ],
        )
    ) == [
        ([0, 1, 2, 3], [0, 1, 2, 3], [0, 1, 2, 3]),
        ([3, 4, 5, 6], [3, 4, 5, 6], [3, 4, 5]),
        ([7, 8, 9],),
    ]


@pytest.mark.parametrize(
    "i, o",
    [
        ("łodź_śmierci", "lodz_smierci"),
        ("Lp. ", "lp"),
        ("Jedenaście    milionów", "jedenascie_milionow"),
        (123.123, "123_123"),
    ],
)
def test_normalize_xlsx_header_column_name(i, o):
    assert normalize_xlsx_header_column_name(i) == o


def test_InputXLSX_rows_as_list():
    fn = curdir("test_file.xlsx", __file__)
    i = InputXLSX(fn, const.IMPORT_MAKSYMALNYCH_SLOTOW_COLUMNS)
    ret = list(i.rows_as_list())
    assert "Kowalski" in ret[0]


def test_InputXLSX_rows_as_dict():
    fn = curdir("test_file.xlsx", __file__)
    i = InputXLSX(fn, const.IMPORT_MAKSYMALNYCH_SLOTOW_COLUMNS)
    ret = list(i.rows_as_dict())
    assert ret[0]["nazwisko"] == "Kowalski"
    assert ret[0]["__nrow__"] == 2


@pytest.mark.parametrize(
    "i, o",
    [
        ("3.2", Decimal("3.2")),
        (3.2, Decimal("3.2")),
        (3.12345, Decimal("3.1235")),
        (float(3.2), Decimal("3.2")),
        (None, None),
        (3, Decimal(3)),
    ],
)
def test_float_or_string_or_int_or_none_to_decimal(i, o):
    assert float_or_string_or_int_or_none_to_decimal(i) == o



import django_filters
from rest_framework import viewsets

from api_v1.serializers.wydawnictwo_zwarte import (
    Wydawnictwo_Zwarte_AutorSerializer,
    Wydawnictwo_Zwarte_StreszczenieSerializer,
    Wydawnictwo_ZwarteSerializer,
)
from api_v1.viewsets.common import StreszczeniaPagination, UkryjStatusyKorektyMixin

from bpp.models import (
    Wydawnictwo_Zwarte,
    Wydawnictwo_Zwarte_Autor,
    Wydawnictwo_Zwarte_Streszczenie,
)


class Wydawnictwo_Zwarte_AutorViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Wydawnictwo_Zwarte_Autor.objects.all().select_related()
    serializer_class = Wydawnictwo_Zwarte_AutorSerializer


class Wydawnictwo_ZwarteFilterSet(django_filters.rest_framework.FilterSet):
    ostatnio_zmieniony = django_filters.DateTimeFromToRangeFilter("ostatnio_zmieniony")
    rok = django_filters.RangeFilter("rok")

    class Meta:
        fields = ["ostatnio_zmieniony", "charakter_formalny", "rok"]
        model = Wydawnictwo_Zwarte


class Wydawnictwo_ZwarteViewSet(
    UkryjStatusyKorektyMixin, viewsets.ReadOnlyModelViewSet
):
    # Lista musi być posortowana po PK aby nie było duplikatów
    queryset = (
        Wydawnictwo_Zwarte.objects.exclude(nie_eksportuj_przez_api=True)
        .order_by("pk")
        .select_related("status_korekty")
        .prefetch_related("autorzy_set", "nagrody", "slowa_kluczowe", "streszczenia")
    )
    serializer_class = Wydawnictwo_ZwarteSerializer
    filterset_class = Wydawnictwo_ZwarteFilterSet


class Wydawnictwo_Zwarte_StreszczenieViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Wydawnictwo_Zwarte_Streszczenie.objects.all()
    serializer_class = Wydawnictwo_Zwarte_StreszczenieSerializer
    pagination_class = StreszczeniaPagination



# Generated by Django 3.0.14 on 2021-09-25 19:34

import os

from django.db import migrations

dirname = os.path.dirname(__file__)


def template_n(elem):
    return f"{dirname}/../templates/{elem}"


def create_template(Template, name):
    Template.objects.create(
        name=name,
        content=open(template_n(name), "r").read(),
    )


def instaluj_szablony(apps, schema_editor):
    Template = apps.get_model("dbtemplates", "Template")

    create_template(Template, "opis_bibliograficzny.html")
    create_template(Template, "browse/praca_tabela.html")

    SzablonDlaOpisuBibliograficznego = apps.get_model(
        "bpp", "SzablonDlaOpisuBibliograficznego"
    )
    SzablonDlaOpisuBibliograficznego.objects.create(
        model=None,
        template=Template.objects.get(name="opis_bibliograficzny.html"),
    )


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0294_szablony_opisu_stron"),
    ]

    operations = [migrations.RunPython(instaluj_szablony, migrations.RunPython.noop)]



import os

import pytest
from django.urls import reverse
from model_bakery import baker

from bpp.models import Uczelnia
from bpp.tests import proper_click_element

from django_bpp.selenium_util import wait_for_page_load


@pytest.mark.django_db(transaction=True)
def test_integracyjny(admin_browser, asgi_live_server):
    baker.make(Uczelnia)
    admin_browser.visit(asgi_live_server.url + reverse("import_dyscyplin:index"))

    with wait_for_page_load(admin_browser):
        admin_browser.find_by_id("add-new-file").click()

    admin_browser.find_by_id("id_plik").type(
        os.path.join(
            os.path.dirname(__file__), "../static/import_dyscyplin/xlsx/default.xlsx"
        )
    )

    with wait_for_page_load(admin_browser):
        admin_browser.find_by_id("id_submit").click()

    btn = admin_browser.find_by_id("submit-id-submit")
    btn[0]._element.location_once_scrolled_into_view

    with wait_for_page_load(admin_browser):
        proper_click_element(admin_browser, btn)
        # btn.click()

    admin_browser.wait_for_condition(lambda browser: "Lubelski" in admin_browser.html)



from django.test import TestCase

from bpp.management.commands.import_bpp import afiliacja


class TestImportBpp(TestCase):
    def test_afiliacja(self):
        kw = {}

        d1 = dict(recenzowana=True, id=0)
        afiliacja(d1, kw)
        self.assertEqual(kw["recenzowana"], True)

        d1 = dict(recenzowana=None, id=0)
        afiliacja(d1, kw)
        self.assertEqual(kw["recenzowana"], False)



from dal import autocomplete
from django import forms
from django.db.models import Q
from djangoql.admin import DjangoQLSearchMixin

from ewaluacja2021.models import IloscUdzialowDlaAutora
from pbn_api.models import Scientist
from ..models import (  # Publikacja_Habilitacyjna
    Autor,
    Autor_Dyscyplina,
    Autor_Jednostka,
    Dyscyplina_Naukowa,
    Jednostka,
)
from .core import BaseBppAdminMixin
from .filters import (
    JednostkaFilter,
    OrcidObecnyFilter,
    PBN_UID_IDObecnyFilter,
    PBNIDObecnyFilter,
)
from .helpers import ADNOTACJE_FIELDSET, CHARMAP_SINGLE_LINE, ZapiszZAdnotacjaMixin

from django.contrib import admin

# Proste tabele

# Autor_Dyscyplina


class IloscUdzialowDlaAutoraInline(admin.TabularInline):
    model = IloscUdzialowDlaAutora
    extra = 1
    fields = ["dyscyplina_naukowa", "ilosc_udzialow", "ilosc_udzialow_monografie"]

    def get_formset(self, request, obj=None, **kwargs):
        formset = super().get_formset(request, obj, **kwargs)

        dyscypliny_autora = Autor_Dyscyplina.objects.filter(autor=obj).values(
            "dyscyplina_naukowa_id",
        )
        subdyscypliny_autora = (
            Autor_Dyscyplina.objects.filter(autor=obj)
            .exclude(subdyscyplina_naukowa=None)
            .values(
                "subdyscyplina_naukowa_id",
            )
        )
        formset.form.base_fields[
            "dyscyplina_naukowa"
        ].queryset = Dyscyplina_Naukowa.objects.filter(
            Q(pk__in=dyscypliny_autora) | Q(pk__in=subdyscypliny_autora)
        )

        return formset


class Autor_DyscyplinaInlineForm(forms.ModelForm):
    dyscyplina_naukowa = forms.ModelChoiceField(
        queryset=Dyscyplina_Naukowa.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:dyscyplina-autocomplete"),
    )

    subdyscyplina_naukowa = forms.ModelChoiceField(
        queryset=Dyscyplina_Naukowa.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:dyscyplina-autocomplete"),
        required=False,
    )

    class Meta:
        fields = "__all__"

    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)
        if kw.get("instance"):
            self.fields["rok"].disabled = True


class Autor_DyscyplinaInline(admin.TabularInline):
    model = Autor_Dyscyplina
    form = Autor_DyscyplinaInlineForm
    extra = 1
    fields = (
        "rok",
        "rodzaj_autora",
        "wymiar_etatu",
        "dyscyplina_naukowa",
        "procent_dyscypliny",
        "subdyscyplina_naukowa",
        "procent_subdyscypliny",
    )


# Autor_Jednostka


class Autor_JednostkaInlineForm(forms.ModelForm):
    autor = forms.ModelChoiceField(
        queryset=Autor.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:autor-autocomplete"),
    )

    jednostka = forms.ModelChoiceField(
        queryset=Jednostka.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:jednostka-autocomplete"),
    )

    class Meta:
        fields = "__all__"


class Autor_JednostkaInline(admin.TabularInline):
    model = Autor_Jednostka
    form = Autor_JednostkaInlineForm
    extra = 1


# Autorzy


class AutorForm(forms.ModelForm):
    pbn_uid = forms.ModelChoiceField(
        label="Odpowiednik w PBN",
        required=False,
        queryset=Scientist.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:scientist-autocomplete", attrs=dict(style="width: 746px;")
        ),
    )

    class Meta:
        fields = "__all__"
        model = Autor
        widgets = {"imiona": CHARMAP_SINGLE_LINE, "nazwisko": CHARMAP_SINGLE_LINE}


class AutorAdmin(
    DjangoQLSearchMixin, ZapiszZAdnotacjaMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    djangoql_completion_enabled_by_default = False
    djangoql_completion = True

    form = AutorForm
    autocomplete_fields = ["pbn_uid"]

    list_display = [
        "nazwisko",
        "imiona",
        "tytul",
        "pseudonim",
        "poprzednie_nazwiska",
        "email",
        "pbn_id",
        "orcid",
        "pbn_uid_id",
    ]
    list_select_related = [
        "tytul",
    ]
    fields = None
    inlines = [
        Autor_JednostkaInline,
        Autor_DyscyplinaInline,
        IloscUdzialowDlaAutoraInline,
    ]
    list_filter = [
        JednostkaFilter,
        "aktualna_jednostka__wydzial",
        "tytul",
        PBNIDObecnyFilter,
        OrcidObecnyFilter,
        PBN_UID_IDObecnyFilter,
    ]
    search_fields = [
        "imiona",
        "nazwisko",
        "pseudonim",
        "poprzednie_nazwiska",
        "email",
        "www",
        "id",
        "pbn_id",
        "system_kadrowy_id",
        "orcid",
        "aktualna_jednostka__nazwa",
    ]
    readonly_fields = ["ostatnio_zmieniony", "aktualna_jednostka"]

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "imiona",
                    "nazwisko",
                    "tytul",
                    "pseudonim",
                    "pokazuj",
                    "email",
                    "www",
                    "orcid",
                    "orcid_w_pbn",
                    "pbn_id",
                    "pbn_uid",
                    "system_kadrowy_id",
                    "aktualna_jednostka",
                )
            },
        ),
        (
            "Biografia",
            {
                "classes": ("grp-collapse grp-closed",),
                "fields": (
                    "urodzony",
                    "zmarl",
                    "poprzednie_nazwiska",
                    "pokazuj_poprzednie_nazwiska",
                ),
            },
        ),
        ADNOTACJE_FIELDSET,
    )


admin.site.register(Autor, AutorAdmin)



DELETED = "DELETED"
ACTIVE = "ACTIVE"

PBN_POST_PUBLICATIONS_URL = "/api/v1/publications"
PBN_GET_LANGUAGES_URL = "/api/v1/dictionary/languages"
PBN_SEARCH_PUBLICATIONS_URL = "/api/v1/search/publications"

PBN_GET_JOURNAL_BY_ID = f"/api/v1/journals/{id}"

DEFAULT_BASE_URL = "https://pbn-micro-alpha.opi.org.pl"

NEEDS_PBN_AUTH_MSG = (
    "W celu poprawnej autentykacji należy podać poprawny token użytkownika aplikacji."
)
PBN_DELETE_PUBLICATION_STATEMENT = (
    "/api/v1/institutionProfile/publications/{publicationId}"
)
PBN_GET_PUBLICATION_BY_ID_URL = "/api/v1/publications/id/{id}"

PBN_GET_INSTITUTION_STATEMENTS = (
    "/api/v1/institutionProfile/publications/page/statements"
)



# Generated by Django 3.0.14 on 2021-10-28 01:41

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0033_sent_data_typ_rekordu"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="publication",
            options={
                "verbose_name": "Publikacja z PBN API",
                "verbose_name_plural": "Publikacje z PBN API",
            },
        ),
        migrations.AddField(
            model_name="oswiadczenieinstytucji",
            name="statedTimestamp",
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="sentdata",
            name="pbn_uid",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="pbn_api.Publication",
                verbose_name="Publikacja z PBN",
            ),
        ),
    ]



# -*- encoding: utf-8 -*-

from django.core.management import BaseCommand
from django.db.models import CharField, TextField, Q
from django.apps import apps
from bpp.models import Sumy, Typ_KBN, Charakter_Formalny
from bpp.util import usun_nieuzywany_typ_charakter


class Command(BaseCommand):
    help = "Usuwa nieuzywane charaktery formalne i typy KBN (stosuj po imporcie z DBF)"

    def add_arguments(self, parser):
        parser.add_argument("--dry-run", action="store_true")

    def handle(self, dry_run=False, *args, **options):
        usun_nieuzywany_typ_charakter(Typ_KBN, "typ_kbn", dry_run)
        usun_nieuzywany_typ_charakter(Charakter_Formalny, "charakter_formalny", dry_run)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-14 15:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0122_import_dyscyplin'),
    ]

    operations = [
        migrations.AddField(
            model_name='dyscyplina_naukowa',
            name='kod',
            field=models.CharField(blank=True, max_length=20, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='dyscyplina_naukowa',
            name='nazwa',
            field=models.CharField(max_length=200, unique=True),
        ),
    ]



from model_bakery import baker

from django.contrib.admin.models import ADDITION, CHANGE, LogEntry
from django.contrib.contenttypes.models import ContentType

from bpp.admin.filters import OstatnioZmienionePrzezFilter, UtworzonePrzezFilter
from bpp.models import Wydawnictwo_Zwarte


def test_OstatnioZmienionePrzezFilter(wydawnictwo_zwarte, admin_user):

    LogEntry.objects.create(
        action_flag=CHANGE,
        object_id=wydawnictwo_zwarte.pk,
        content_type_id=ContentType.objects.get_for_model(wydawnictwo_zwarte).pk,
        user=admin_user,
    )

    f = OstatnioZmienionePrzezFilter(None, {}, Wydawnictwo_Zwarte, None)

    f.value = lambda *args, **kw: admin_user.pk

    assert wydawnictwo_zwarte in f.queryset(None, Wydawnictwo_Zwarte.objects.all())

    assert admin_user.pk in [x[0] for x in f.lookups(None, None)]


def test_UtworzonePrzeZFilter(wydawnictwo_zwarte, admin_user, normal_django_user):
    drugie_wydawnictwo_zwarte = baker.make(Wydawnictwo_Zwarte)
    content_type_id = ContentType.objects.get_for_model(wydawnictwo_zwarte).pk

    LogEntry.objects.create(
        action_flag=ADDITION,
        object_id=wydawnictwo_zwarte.pk,
        user=admin_user,
        content_type_id=content_type_id,
    )

    LogEntry.objects.create(
        action_flag=ADDITION,
        object_id=drugie_wydawnictwo_zwarte.pk,
        user=normal_django_user,
        content_type_id=content_type_id,
    )

    f = UtworzonePrzezFilter(None, {}, Wydawnictwo_Zwarte, None)

    f.value = lambda *args, **kw: admin_user.pk

    assert wydawnictwo_zwarte in f.queryset(None, Wydawnictwo_Zwarte.objects.all())
    assert drugie_wydawnictwo_zwarte not in f.queryset(
        None, Wydawnictwo_Zwarte.objects.all()
    )

    user_ids = [x[0] for x in f.lookups(None, None)]
    assert admin_user.pk in user_ids



# Generated by Django 2.1.13 on 2019-11-07 07:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0191_wydawnictwo_zwarte_zewnetrzna_baza_danych'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='dyscyplina_naukowa',
            name='dyscyplina_nadrzedna',
        ),
        migrations.RemoveField(
            model_name='dyscyplina_naukowa',
            name='level',
        ),
        migrations.RemoveField(
            model_name='dyscyplina_naukowa',
            name='lft',
        ),
        migrations.RemoveField(
            model_name='dyscyplina_naukowa',
            name='rght',
        ),
        migrations.RemoveField(
            model_name='dyscyplina_naukowa',
            name='tree_id',
        ),
        migrations.AlterField(
            model_name='dyscyplina_naukowa',
            name='kod',
            field=models.CharField(default='0.00', max_length=20, unique=True),
            preserve_default=False,
        ),
    ]



import json
import multiprocessing
import os
import re
import sys
from datetime import datetime, timedelta
from math import ceil, floor
from pathlib import Path
from typing import Dict, List

import bleach
import lxml.html
import openpyxl.worksheet.worksheet
import progressbar
from django.apps import apps
from django.conf import settings
from django.db.models import Max, Min
from openpyxl.utils import get_column_letter
from openpyxl.worksheet.filters import AutoFilter
from openpyxl.worksheet.table import Table, TableColumn, TableStyleInfo
from psycopg2.extensions import QuotedString
from unidecode import unidecode

from django.utils import timezone
from django.utils.html import strip_tags

non_url = re.compile(r"[^\w-]+")


def get_fixture(name):
    p = Path(__file__).parent / "fixtures" / ("%s.json" % name)
    ret = json.load(open(p, "rb"))
    ret = [x["fields"] for x in ret if x["model"] == ("bpp.%s" % name)]
    return {x["skrot"].lower().strip(): x for x in ret}


def fulltext_tokenize(s):
    s = (
        s.replace(":", " ")
        .replace("*", " ")
        .replace('"', " ")
        .replace("|", " ")
        .replace("'", " ")
        .replace("&", " ")
        .replace("\\", " ")
        .replace("(", " ")
        .replace(")", " ")
        .replace("[", " ")
        .replace("]", " ")
        .replace("\t", " ")
        .replace("\n", " ")
        .replace("\r", " ")
        .replace("<", " ")
        .replace(">", " ")
    )
    return [x.strip() for x in s.split(" ") if x.strip()]


class FulltextSearchMixin:
    fts_field = "search"

    def fulltext_filter(self, qstr):
        def quotes(wordlist):
            ret = []
            for elem in wordlist:
                ret.append(str(QuotedString(elem.replace("\\", "").encode("utf-8"))))
            return ret

        def startswith(wordlist):
            return [x + ":*" for x in quotes(wordlist)]

        def negative(wordlist):
            return ["!" + x for x in startswith(wordlist)]

        if qstr is None:
            qstr = ""

        if isinstance(qstr, bytes):
            qstr = qstr.decode("utf-8")

        clean_qstr = strip_tags(qstr)
        words = fulltext_tokenize(clean_qstr)
        qstr = " & ".join(startswith(words))
        params = ("bpp_nazwy_wlasne", qstr)

        return self.all().extra(
            select={
                self.model._meta.db_table
                + "__rank": "ts_rank_cd("
                + self.model._meta.db_table
                + "."
                + self.fts_field
                + ", to_tsquery(%s::regconfig, %s), 16)"
            },
            select_params=params,
            where=[
                self.model._meta.db_table
                + "."
                + self.fts_field
                + " @@ to_tsquery(%s::regconfig, %s)"
            ],
            params=params,
            order_by=["-" + self.model._meta.db_table + "__rank"],
        )


def strip_html(s):
    if not s:
        return s

    return lxml.html.fromstring(str(s)).text_content()


def slugify_function(s):
    s = unidecode(strip_html(s)).replace(" ", "-")
    while s.find("--") >= 0:
        s = s.replace("--", "-")
    return non_url.sub("", s)


def get_original_object(object_name, object_pk):
    from bpp.models import TABLE_TO_MODEL

    klass = TABLE_TO_MODEL.get(object_name)
    try:
        return klass.objects.get(pk=object_pk)
    except klass.DoesNotExist:
        return


def get_copy_from_db(instance):
    if not instance.pk:
        return None
    return instance.__class__._default_manager.get(pk=instance.pk)


def has_changed(instance, field_or_fields):
    try:
        original = get_copy_from_db(instance)
    except instance.__class__.DoesNotExist:
        return True
        # Jeżeli w bazie danych nie ma tego obiektu, no to bankowo
        # się zmienił...
        return True

    fields = field_or_fields
    if isinstance(field_or_fields, str):
        fields = [field_or_fields]

    for field in fields:
        if not getattr(instance, field) == getattr(original, field):
            return True

    return False


class Getter:
    """Klasa pomocnicza dla takich danych jak Typ_KBN czy
    Charakter_Formalny, umozliwia po zainicjowaniu pobieranie
    tych klas po atrybucie w taki sposob:

    >>> kbn = Getter(Typ_KBN)
    >>> kbn.PO == Typ_KBN.objects.get(skrot='PO')
    True
    """

    def __init__(self, klass, field="skrot"):
        self.field = field
        self.klass = klass

    def __getitem__(self, item):
        kw = {self.field: item}
        return self.klass.objects.get(**kw)

    __getattr__ = __getitem__


class NewGetter(Getter):
    """Zwraca KeyError zamiast DoesNotExist."""

    def __getitem__(self, item):
        kw = {self.field: item}
        try:
            return self.klass.objects.get(**kw)
        except self.klass.DoesNotExist as e:
            raise KeyError(e)

    __getattr__ = __getitem__


def zrob_cache(t):
    zle_znaki = [
        " ",
        ":",
        ";",
        "-",
        ",",
        "-",
        ".",
        "(",
        ")",
        "?",
        "!",
        "ę",
        "ą",
        "ł",
        "ń",
        "ó",
        "ź",
        "ż",
    ]
    for znak in zle_znaki:
        t = t.replace(znak, "")
    return t.lower()


def remove_old_objects(klass, file_field="file", field_name="created_on", days=7):
    since = datetime.now() - timedelta(days=days)

    kwargs = {}
    kwargs["%s__lt" % field_name] = since

    for rec in klass.objects.filter(**kwargs):
        try:
            path = getattr(rec, file_field).path
        except ValueError:
            path = None

        rec.delete()

        if path is not None:
            try:
                os.unlink(path)
            except OSError:
                pass


def rebuild_contenttypes():
    app_config = apps.get_app_config("bpp")
    from django.contrib.contenttypes.management import create_contenttypes

    create_contenttypes(app_config, verbosity=0)


#
# Progress bar
#


def pbar(query, count=None, label="Progres...", disable_progress_bar=False):
    if sys.stdout.isatty() and not disable_progress_bar:
        if count is None:
            if hasattr(query, "count"):
                try:
                    count = query.count()
                except TypeError:
                    count = len(query)
            elif hasattr(query, "__len__"):
                count = len(query)

        return progressbar.progressbar(
            query,
            max_value=count,
            widgets=[
                progressbar.FormatLabel(label),
                " ",
                progressbar.AnimatedMarker(),
                " ",
                progressbar.SimpleProgress(),
                " ",
                progressbar.Timer(),
                " ",
                progressbar.ETA(),
            ],
        )
    else:
        # You're being piped or redirected
        return query


#
# Multiprocessing stuff
#


def partition(min, max, num_proc, fun=ceil):
    s = int(fun((max - min) / num_proc))
    cnt = min
    ret = []
    while cnt < max:
        ret.append((cnt, cnt + s))
        cnt += s
    return ret


def partition_ids(model, num_proc, attr="idt"):
    d = model.objects.aggregate(min=Min(attr), max=Max(attr))
    return partition(d["min"], d["max"], num_proc)


def partition_count(objects, num_proc):
    return partition(0, objects.count(), num_proc, fun=ceil)


def no_threads(multiplier=0.75):
    return max(int(floor(multiprocessing.cpu_count() * multiplier)), 1)


class safe_html_defaults:
    ALLOWED_TAGS = (
        "a",
        "abbr",
        "acronym",
        "b",
        "blockquote",
        "code",
        "em",
        "i",
        "li",
        "ol",
        "strong",
        "ul",
        "font",
        "div",
        "span",
        "br",
        "strike",
        "h2",
        "h3",
        "h4",
        "h5",
        "h6",
        "p",
        "table",
        "tr",
        "td",
        "th",
        "thead",
        "tbody",
        "dl",
        "dd",
        "u",
    )

    ALLOWED_ATTRIBUTES = {
        "*": ["class"],
        "a": ["href", "title", "rel"],
        "abbr": ["title"],
        "acronym": ["title"],
        "font": [
            "face",
            "size",
        ],
        "div": [
            "style",
        ],
        "span": [
            "style",
        ],
        "ul": [
            "style",
        ],
    }


def safe_html(html):
    html = html or ""

    ALLOWED_TAGS = getattr(settings, "ALLOWED_TAGS", safe_html_defaults.ALLOWED_TAGS)
    ALLOWED_ATTRIBUTES = getattr(
        settings, "ALLOWED_ATTRIBUTES", safe_html_defaults.ALLOWED_ATTRIBUTES
    )
    STRIP_TAGS = getattr(settings, "STRIP_TAGS", True)
    return bleach.clean(
        html,
        tags=ALLOWED_TAGS,
        attributes=ALLOWED_ATTRIBUTES,
        strip=STRIP_TAGS,
    )


def set_seq(s):
    if settings.DATABASES["default"]["ENGINE"].find("postgresql") >= 0:
        from django.db import connection

        cursor = connection.cursor()
        cursor.execute(f"SELECT setval('{s}_id_seq', (SELECT MAX(id) FROM {s}))")


def usun_nieuzywany_typ_charakter(klass, field, dry_run):
    from bpp.models import Rekord

    for elem in klass.objects.all():
        kw = {field: elem}
        if not Rekord.objects.filter(**kw).exists():
            print(f"Kasuje {elem}")
            if not dry_run:
                elem.delete()


isbn_regex = re.compile(
    r"^isbn\s*[0-9]*[-| ][0-9]*[-| ][0-9]*[-| ][0-9]*[-| ][0-9]*X?",
    flags=re.IGNORECASE,
)


def wytnij_isbn_z_uwag(uwagi):
    if uwagi is None:
        return

    if uwagi == "":
        return

    if uwagi.lower().find("isbn-10") >= 0 or uwagi.lower().find("isbn-13") >= 0:
        return None

    res = isbn_regex.search(uwagi)
    if res:
        res = res.group()
        isbn = res.replace("ISBN", "").replace("isbn", "").strip()
        reszta = uwagi.replace(res, "").strip()

        while (
            reszta.startswith(".") or reszta.startswith(";") or reszta.startswith(",")
        ):
            reszta = reszta[1:].strip()

        return isbn, reszta


def crispy_form_html(self, key):
    from crispy_forms_foundation.layout import HTML, Column, Row

    from django.utils.functional import lazy

    def _():
        return self.initial.get(key, None) or ""

    return Row(Column(HTML(lazy(_, str)())))


def formdefaults_html_before(form):
    return crispy_form_html(form, "formdefaults_pre_html")


def formdefaults_html_after(form):
    return crispy_form_html(form, "formdefaults_post_html")


def knapsack(W, wt, val, ids, zwracaj_liste_przedmiotow=True):
    """
    :param W: wielkosc plecaka -- maksymalna masa przedmiotów w plecaku (zbierany slot)
    :param wt: masy przedmiotów, które można włożyć do plecaka (sloty prac)
    :param val: ceny przedmiotów, które można włożyc do plecaka (punkty PKdAut prac)
    :param ids: ID prac, które można włożyć do plecaka (rekord.pk)
    :param zwracaj_liste_przedmiotow: gdy True (domyślnie) funkcja zwróci listę z identyfikatorami włożonych
    przedmiotów, gdy False zwrócona lista będzie pusta

    :returns: tuple(mp, lista), gdzie mp to maksymalna możliwa wartość włożonych przedmiotów, a lista to lista
    lub pusta lista gdy parametr `zwracaj_liste_przemiotów` był pusty
    """

    assert len(wt) == len(val) == len(ids), "Listy są różnej długości"

    sum_wt = sum(wt)
    if sum_wt <= W:
        # Jeżeli wszystkie przedmioty zmieszczą się w plecaku, to po co liczyć cokolwiek
        if zwracaj_liste_przedmiotow:
            return sum(val), ids
        return sum(val), []

    n = len(wt)

    K = [[0 for x in range(W + 1)] for x in range(n + 1)]

    for i in range(n + 1):
        for w in range(W + 1):
            if i == 0 or w == 0:
                K[i][w] = 0
            elif wt[i - 1] <= w:
                K[i][w] = max(val[i - 1] + K[i - 1][w - wt[i - 1]], K[i - 1][w])
            else:
                K[i][w] = K[i - 1][w]

    res = maks_punkty = K[n][W]
    lista = []

    if zwracaj_liste_przedmiotow:
        w = W
        for i in range(n, 0, -1):
            if res <= 0:
                break

            if res == K[i - 1][w]:
                continue
            else:
                lista.append(ids[i - 1])

                res = res - val[i - 1]
                w = w - wt[i - 1]

    return maks_punkty, lista


DEC2INT = 10000


def intsack(W, wt, val, ids):
    pkt, ids = knapsack(
        int(W * DEC2INT),
        [int(x * DEC2INT) for x in wt],
        [int(x * DEC2INT) for x in val],
        ids,
    )
    return pkt / DEC2INT, ids


def disable_multithreading_by_monkeypatching_pool(pool):
    def apply(fun, args=()):
        return fun(*args)

    pool.apply = apply

    def starmap(fun, lst):
        for elem in lst:
            fun(*elem)

    pool.starmap = starmap


def year_last_month():
    now = timezone.now().date()
    if now.month >= 2:
        return now.year
    return now.year - 1


#
# Seq Scan check
#


class PerformanceFailure(Exception):
    pass


def fail_if_seq_scan(qset, DEBUG):
    """
    Funkcja weryfikujaca, czy w wyjasnieniu zapytania (EXPLAIN) nie wystapi ciag znakow 'Seq Scan',
    jezeli tak to wyjatek PerformanceFailure z zapytaniem + wyjasnieniem
    """
    if DEBUG:
        explain = qset.explain()
        if explain.find("Seq Scan") >= 0:
            print("\r\n", explain)
            raise PerformanceFailure(str(qset.query), explain)


def rebuild_instances_of_models(modele, *args, **kw):
    from denorm import denorms
    from django.db import transaction

    with transaction.atomic():
        for model in modele:
            denorms.rebuild_instances_of(model, *args, **kw)


def worksheet_columns_autosize(
    ws: openpyxl.worksheet.worksheet.Worksheet,
    max_width: int = 55,
    column_widths: Dict[str, int] | None = None,
    dont_resize_those_columns: List[int] | None = None,
    right_margin=2,
    multiplier=1.1,
):

    if column_widths is None:
        column_widths = {}

    if dont_resize_those_columns is None:
        dont_resize_those_columns = []

    for ncol, col in enumerate(ws.columns):
        max_length = 0
        column = col[0].column_letter  # Get the column name

        # Nie ustawiaj szerokosci tym kolumnom, one będą jako auto-size
        if ncol in dont_resize_those_columns:
            continue

        if column in column_widths:
            adjusted_width = column_widths[column]
        else:
            for cell in col:
                if cell.value is None or not str(cell.value):
                    continue

                text = str(cell.value)

                if text.startswith("=HYPERLINK"):
                    try:
                        # Wyciągnij z hiperlinku jego faktyczny opis tekstowy na cele
                        # liczenia szerokości kolumny
                        text = text.split('"')[3]
                    except IndexError:
                        pass

                max_line_len = max(len(line) for line in text.split("\n"))
                max_length = max(max_length, max_line_len)

            adjusted_width = (max_length + right_margin) * multiplier
            if adjusted_width > max_width:
                adjusted_width = max_width

        ws.column_dimensions[column].width = adjusted_width


def worksheet_create_table(
    ws: openpyxl.worksheet.worksheet.Worksheet,
    title="Tabela",
    first_table_row=1,
    totals=False,
    table_columns=None,
):
    """
    Formatuje skoroszyt jako tabelę.

    :param first_table_row: pierwszy wiersz tabeli (licząc od nagłówka)

    :param table_columns: określa rodzaj kolumn w tabeli, jeżeli None to tytuły nagłówków zostaną pobrane
    z pierwszego wiersza w arkuszu.
    """
    max_column = ws.max_column
    max_column_letter = get_column_letter(max_column)
    max_row = ws.max_row

    style = TableStyleInfo(
        name="TableStyleMedium9",
        showFirstColumn=False,
        showLastColumn=False,
        showRowStripes=True,
        showColumnStripes=True,
    )

    if table_columns is None:
        table_columns = tuple(
            TableColumn(id=h, name=header.value)
            for h, header in enumerate(next(iter(ws.rows), None), start=1)
        )

    tab = Table(
        displayName=title,
        ref=f"A{first_table_row}:{max_column_letter}{max_row}",
        autoFilter=AutoFilter(
            ref=f"A{first_table_row}:{max_column_letter}{max_row - 1}"
        ),
        totalsRowShown=True if totals else False,
        totalsRowCount=1 if totals else False,
        tableStyleInfo=style,
        tableColumns=table_columns,
    )

    ws.add_table(tab)


def worksheet_create_urls(
    ws: openpyxl.worksheet.worksheet.Worksheet, default_link_name: str = "[link]"
):
    """Tworzy adresy URL w postaci klikalnego linku z domyslnym tekstem."""

    for column_cell in ws.iter_cols(1, ws.max_column):  # iterate column cell
        if hasattr(column_cell[0].value, "endswith") and column_cell[0].value.endswith(
            "_url"
        ):
            for data in column_cell[1:]:
                if data.value:
                    data.value = '=HYPERLINK("{}", "{}")'.format(
                        data.value, default_link_name
                    )



import pytest

from bpp.models.sloty.core import ISlot
from bpp.models.sloty.wydawnictwo_zwarte import (
    SlotKalkulator_Wydawnictwo_Zwarte_Prog1,
    SlotKalkulator_Wydawnictwo_Zwarte_Prog2,
    SlotKalkulator_Wydawnictwo_Zwarte_Prog3,
)


@pytest.mark.django_db
def test_ISlot_tlumaczenie_tier3(zwarte_z_dyscyplinami):
    rok = 2021
    zwarte_z_dyscyplinami.rok = rok
    i = ISlot(zwarte_z_dyscyplinami)
    zwarte_z_dyscyplinami.punkty_kbn = 2.5
    assert isinstance(i, SlotKalkulator_Wydawnictwo_Zwarte_Prog3)


@pytest.mark.django_db
def test_ISlot_tlumaczenie_tier2(zwarte_z_dyscyplinami, wydawca):
    rok = 2021
    wydawca.poziom_wydawcy_set.create(rok=rok, poziom=1)
    zwarte_z_dyscyplinami.punkty_kbn = 40
    zwarte_z_dyscyplinami.rok = rok
    i = ISlot(zwarte_z_dyscyplinami)
    assert isinstance(i, SlotKalkulator_Wydawnictwo_Zwarte_Prog2)


@pytest.mark.django_db
def test_ISlot_tlumaczenie_tier1(zwarte_z_dyscyplinami, wydawca):
    rok = 2021
    wydawca.poziom_wydawcy_set.create(rok=rok, poziom=2)
    zwarte_z_dyscyplinami.punkty_kbn = 100
    zwarte_z_dyscyplinami.rok = rok
    i = ISlot(zwarte_z_dyscyplinami)
    assert isinstance(i, SlotKalkulator_Wydawnictwo_Zwarte_Prog1)



import sys

from django.core.management import BaseCommand

from tee import core


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("command_name", help="Django command to execute")
        parser.add_argument("otherthings", nargs="*")  # parser.add

    def handle(self, command_name, otherthings, *args, **options):
        new_argv = [sys.argv[0], command_name] + otherthings

        kwargs = {}

        for elem in ["stdout", "stderr"]:
            if options.get(elem):
                kwargs[elem] = options.get(elem)

        core.execute(new_argv, **kwargs)



class WydawnictwoNadrzednePBNAdapter:
    def __init__(self, original):
        self.original = original

    def pbn_get_json(self):
        if self.original.pbn_uid_id is not None:
            return {"objectId": self.original.pbn_uid_id}

        ret = {}

        for attr in "isbn", "issn", "title", "year":
            if hasattr(self.original, attr):
                v = getattr(self.original, attr)
                if v is not None:
                    ret[attr] = v

        ret["title"] = self.original.tytul_oryginalny
        ret["year"] = self.original.rok

        from pbn_api.adapters.wydawnictwo import WydawnictwoPBNAdapter

        volume = WydawnictwoPBNAdapter(self.original).nr_tomu()
        if volume:
            ret["volume"] = volume

        translation = WydawnictwoPBNAdapter(self.original).get_translation()
        ret["translation"] = translation

        return ret



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-16 11:03
from __future__ import unicode_literals

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0127_auto_20180416_1303'),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0128_zewnetrzne_bazy_view")
        ),
    ]



# -*- encoding: utf-8 -*-
import logging
import re

from braces.views import LoginRequiredMixin
from django import forms
from django.utils.decorators import method_decorator
from django.views.generic.base import View

from bpp.models.wydawnictwo_zwarte import MIEJSCE_I_ROK_MAX_LENGTH, Wydawnictwo_Zwarte
from bpp.decorators import json_view

logger = logging.getLogger(__name__)

BRAK_DANYCH = {"rok": "b/d"}


class ValidationFormWydawnictwoZwarte(forms.Form):
    miejsce_i_rok = forms.CharField(required=False, max_length=MIEJSCE_I_ROK_MAX_LENGTH)

    wydawnictwo_nadrzedne = forms.ModelChoiceField(
        Wydawnictwo_Zwarte.objects.all(), required=False
    )


class ValidationFormWydawnictwoCiagle(forms.Form):
    informacje = forms.CharField(required=True)


class ApiJsonView(LoginRequiredMixin, View):
    @method_decorator(json_view)
    def post(self, request, *args, **kwargs):
        form = self.validation_form_class(request.POST)
        if not form.is_valid():
            return {"error": "form", "message": form.errors.as_json()}
        return self.get_data(form.cleaned_data)


class ApiUzupelnijRokWydawnictwoZwarteView(ApiJsonView):
    validation_form_class = ValidationFormWydawnictwoZwarte

    def get_data(self, data):
        if data.get("miejsce_i_rok"):
            try:
                rok = re.search(r"\d{4}", data["miejsce_i_rok"]).group(0)
                return {"rok": rok}
            except (IndexError, AttributeError):
                pass

        if data.get("wydawnictwo_nadrzedne"):
            return {"rok": data["wydawnictwo_nadrzedne"].rok}

        return BRAK_DANYCH


class ApiUzupelnijRokWydawnictwoCiagleView(ApiJsonView):
    validation_form_class = ValidationFormWydawnictwoCiagle

    def get_data(self, data):
        try:
            rok = re.search(r"\d{4}", data["informacje"]).group(0)
            return {"rok": rok}
        except (IndexError, AttributeError):
            pass

        return BRAK_DANYCH



from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models import JSONField

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from bpp import const
from bpp.models import LinkDoPBNMixin


class SentDataManager(models.Manager):
    def get_for_rec(self, rec):
        return self.get(
            object_id=rec.pk, content_type=ContentType.objects.get_for_model(rec)
        )

    def check_if_needed(self, rec, data: dict):
        try:
            sd = self.get_for_rec(rec)
        except SentData.DoesNotExist:
            return True

        if sd.data_sent != data:
            return True

        if not sd.uploaded_okay:
            return True

        return False

    def updated(
        self, rec, data: dict, pbn_uid_id=None, uploaded_okay=True, exception=None
    ):
        try:
            sd = self.get_for_rec(rec)
        except SentData.DoesNotExist:
            self.create(
                object=rec,
                data_sent=data,
                uploaded_okay=uploaded_okay,
                pbn_uid_id=pbn_uid_id,
                exception=exception,
            )
            return

        sd.data_sent = data
        sd.uploaded_okay = uploaded_okay
        sd.exception = exception
        sd.pbn_uid_id = pbn_uid_id
        sd.save()

    def ids_for_model(self, model):
        return self.filter(content_type=ContentType.objects.get_for_model(model))

    def bad_uploads(self, model):
        return (
            self.ids_for_model(model)
            .filter(uploaded_okay=False)
            .values_list("object_id", flat=True)
            .distinct()
        )


class SentData(LinkDoPBNMixin, models.Model):
    url_do_pbn = const.LINK_PBN_DO_PUBLIKACJI
    content_type = models.ForeignKey(
        "contenttypes.ContentType", on_delete=models.CASCADE
    )
    object_id = models.PositiveIntegerField(db_index=True)

    object = GenericForeignKey()

    data_sent = JSONField("Wysłane dane")
    last_updated_on = models.DateTimeField("Data operacji", auto_now=True)

    uploaded_okay = models.BooleanField(
        "Wysłano poprawnie", default=True, db_index=True
    )
    exception = models.TextField("Kod błędu", max_length=65535, blank=True, null=True)

    pbn_uid = models.ForeignKey(
        "pbn_api.Publication",
        verbose_name="Publikacja z PBN",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
    )

    typ_rekordu = models.CharField(max_length=50, blank=True, null=True)

    objects = SentDataManager()

    class Meta:
        verbose_name = "Informacja o wysłanych danych"
        verbose_name_plural = "Informacje o wysłanych danych"

    object.verbose_name = "Rekord"

    def __str__(self):
        return (
            f"Informacja o wysłanych do PBN danych dla rekordu ({self.content_type_id},{self.object_id}) "
            f"z dnia {self.last_updated_on} (status: {'OK' if self.uploaded_okay else 'ERR'})"
        )

    def link_do_pbn_wartosc_id(self):
        return self.pbn_uid_id

    def rekord_w_bpp(self):
        try:
            return self.object
        except ObjectDoesNotExist:
            pass

    def save(
        self, force_insert=False, force_update=False, using=None, update_fields=None
    ):

        if update_fields and "data_sent" in update_fields:
            if self.typ_rekordu != self.data_sent.get("type"):
                update_fields.append("typ_rekordu")

        self.typ_rekordu = self.data_sent.get("type")
        return super().save(
            force_insert=force_insert,
            force_update=force_update,
            using=using,
            update_fields=update_fields,
        )



# Generated by Django 3.0.9 on 2020-08-12 17:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("bpp", "0222_auto_20200812_1927"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="grant_rekordu",
            options={
                "verbose_name": "grant rekordu",
                "verbose_name_plural": "granty rekordu",
            },
        ),
        migrations.AlterUniqueTogether(
            name="grant_rekordu",
            unique_together={("grant", "content_type", "object_id")},
        ),
    ]



# Generated by Django 2.2.10 on 2020-02-23 12:39

import bpp.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0195_cc0"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="pokazuj_raport_slotow_zerowy",
            field=bpp.models.fields.OpcjaWyswietlaniaField(
                choices=[
                    ("always", "zawsze"),
                    ("logged-in", "tylko dla zalogowanych"),
                    ("never", "nigdy"),
                ],
                default="logged-in",
                max_length=50,
                verbose_name="Pokazuj raport slotów zerowy",
            ),
        ),
    ]



from rest_framework import serializers
from taggit_serializer.serializers import TagListSerializerField

from api_v1.serializers.util import (
    AbsoluteUrlSerializerMixin,
    WydawnictwoSerializerMixin,
)
from bpp.models import Praca_Doktorska


class Praca_DoktorskaSerializer(
    AbsoluteUrlSerializerMixin,
    WydawnictwoSerializerMixin,
    serializers.HyperlinkedModelSerializer,
):
    autor = serializers.HyperlinkedRelatedField(
        view_name="api_v1:autor-detail", read_only=True
    )

    promotor = serializers.HyperlinkedRelatedField(
        view_name="api_v1:autor-detail", read_only=True
    )

    jednostka = serializers.HyperlinkedRelatedField(
        view_name="api_v1:jednostka-detail", read_only=True
    )

    slowa_kluczowe = TagListSerializerField()

    class Meta:
        model = Praca_Doktorska
        fields = [
            "id",
            #
            "tytul_oryginalny",
            "tytul",
            #
            "rok",
            "status_korekty",
            #
            "jezyk",
            "jezyk_alt",
            "charakter_formalny",
            "typ_kbn",
            #
            "www",
            "dostep_dnia",
            "public_www",
            "public_dostep_dnia",
            #
            "pubmed_id",
            "pmc_id",
            "doi",
            "pbn_id",
            #
            "impact_factor",
            "punkty_kbn",
            #
            "informacje",
            "szczegoly",
            "uwagi",
            "slowa_kluczowe",
            "strony",
            "tom",
            #
            "liczba_cytowan",
            #
            "openaccess_tryb_dostepu",
            "openaccess_wersja_tekstu",
            "openaccess_licencja",
            "openaccess_czas_publikacji",
            #
            "utworzono",
            "ostatnio_zmieniony",
            #
            "absolute_url",
            #
            "autor",
            "jednostka",
            #
            "promotor",
            "oznaczenie_wydania",
            "miejsce_i_rok",
            "wydawca",
            "wydawca_opis",
        ]



import pytest
from django.core.files.base import ContentFile
from django.db import transaction
from django.db.models import Max

import notifications.core as notifications_core
from import_dyscyplin.models import Import_Dyscyplin
from import_dyscyplin.tasks import przeanalizuj_import_dyscyplin
from notifications.models import Notification

from bpp.models import Autor_Dyscyplina


def test_kasowanie_calosci(
    test4_kasowanie_xlsx,
    normal_django_user,
    mocker,
    transactional_db,
    autor_jan_kowalski,
    rok,
    dyscyplina1,
):
    web_page_uid = "foobar_uid"

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski, rok=rok, dyscyplina_naukowa=dyscyplina1
    )

    with transaction.atomic():
        i = Import_Dyscyplin.objects.create(
            owner=normal_django_user, web_page_uid=web_page_uid, rok=rok
        )
        i.plik.save("test1.xlsx", ContentFile(open(test4_kasowanie_xlsx, "rb").read()))
        i.plik.path

    mocker.patch("notifications.core._send")
    przeanalizuj_import_dyscyplin.delay(i.pk)

    i = Import_Dyscyplin.objects.first()
    i.stworz_kolumny()
    i.zatwierdz_kolumny()
    i.przeanalizuj()
    i.integruj_dyscypliny()
    i.integruj_wiersze()

    with pytest.raises(Autor_Dyscyplina.DoesNotExist):
        Autor_Dyscyplina.objects.get(autor=autor_jan_kowalski, rok=rok)


def test_kasowanie_subdyscypliny(
    test5_kasowanie_subdyscypliny,
    normal_django_user,
    mocker,
    transactional_db,
    autor_jan_kowalski,
    rok,
    dyscyplina1,
    dyscyplina2,
):
    web_page_uid = "foobar_uid"

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_kowalski,
        rok=rok,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    with transaction.atomic():
        i = Import_Dyscyplin.objects.create(
            owner=normal_django_user, web_page_uid=web_page_uid, rok=rok
        )
        i.plik.save(
            "test1.xlsx", ContentFile(open(test5_kasowanie_subdyscypliny, "rb").read())
        )
        i.plik.path

    mocker.patch("notifications.core._send")
    przeanalizuj_import_dyscyplin.delay(i.pk)

    i = Import_Dyscyplin.objects.first()
    i.stworz_kolumny()
    i.zatwierdz_kolumny()
    i.przeanalizuj()
    i.integruj_dyscypliny()
    i.integruj_wiersze()

    ad = Autor_Dyscyplina.objects.get(autor=autor_jan_kowalski, rok=rok)
    assert ad.subdyscyplina_naukowa is None


def test_przeanalizuj_import_dyscyplin(
    test1_xlsx, normal_django_user, mocker, transactional_db
):

    web_page_uid = "foobar_uid"

    with transaction.atomic():
        i = Import_Dyscyplin.objects.create(
            owner=normal_django_user, web_page_uid=web_page_uid
        )
        i.plik.save("test1.xls", ContentFile(open(test1_xlsx, "rb").read()))
        i.plik.path

    mocker.patch("notifications.core._send")

    przeanalizuj_import_dyscyplin.delay(i.pk)

    link = f"/import_dyscyplin/detail/{i.pk}/?notification=1"

    notifications_core._send.assert_called_once_with(
        f"import_dyscyplin.import_dyscyplin-{i.pk}",
        {"id": Notification.objects.all().aggregate(x=Max("pk"))["x"], "url": link},
    )



# Generated by Django 3.0.14 on 2021-11-09 23:00

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0309_auto_20211110_0000"),
        ("ewaluacja2021", "0005_auto_20211028_0039"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="importmaksymalnychslotow",
            options={"ordering": ("-ostatnia_zmiana",)},
        ),
        migrations.CreateModel(
            name="ZamowienieNaRaport",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "rodzaj",
                    models.CharField(
                        choices=[
                            ("plecakowy", "plecakowy"),
                            ("genetyczny", "genetyczny"),
                        ],
                        max_length=20,
                        verbose_name="Rodzaj algorytmu",
                    ),
                ),
                ("uid_zadania", models.TextField(blank=True, null=True)),
                ("plik_wyjsciowy", models.FileField(upload_to="")),
                ("wykres_wyjsciowy", models.ImageField(upload_to="")),
                ("ostatnio_zmodyfikowany", models.DateTimeField(auto_now=True)),
                (
                    "dyscyplina_naukowa",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="bpp.Dyscyplina_Naukowa",
                    ),
                ),
            ],
        ),
    ]



from django.db import models
from django.db.models import JSONField

from notifications import core
from notifications.core import convert_obj_to_channel_name


class NotificationManager(models.Manager):
    def send_redirect(self, obj, url):
        res = self.create(
            channel_name=convert_obj_to_channel_name(obj), values={"url": url}
        )
        res.send()

    def unacknowledged(self):
        return self.filter(acknowledged=False)

    def on_connect(self, channels):
        """Send all unacknowledged notifications to this user/channel"""
        for notification in self.unacknowledged().filter(channel_name__in=channels):
            notification.send()


class Notification(models.Model):
    channel_name = models.CharField(max_length=128, db_index=True)
    created_on = models.DateTimeField(auto_now_add=True)

    values = JSONField()

    acknowledged = models.BooleanField(default=False, db_index=True)

    objects = NotificationManager()

    class Meta:
        ordering = ("created_on",)

    def send(self):
        return core._send(self.channel_name, dict(id=self.pk, **self.values))

    def acknowledge(self):
        self.acknowledged = True
        self.save()



from django.db import transaction

from ... import const
from ...const import PBN_MAX_ROK, PBN_MIN_ROK
from .exceptions import CannotAdapt
from .wydawnictwo_ciagle import (
    SlotKalkulator_Wydawnictwo_Ciagle_Prog1,
    SlotKalkulator_Wydawnictwo_Ciagle_Prog2,
)

from django.contrib.contenttypes.models import ContentType

from django.utils.functional import cached_property

from bpp.models import Typ_Odpowiedzialnosci, Uczelnia, Wydawca
from bpp.models.cache import Cache_Punktacja_Autora, Cache_Punktacja_Dyscypliny
from bpp.models.patent import Patent
from bpp.models.sloty.wydawnictwo_ciagle import SlotKalkulator_Wydawnictwo_Ciagle_Prog3
from bpp.models.sloty.wydawnictwo_zwarte import (
    SlotKalkulator_Wydawnictwo_Zwarte_Prog1,
    SlotKalkulator_Wydawnictwo_Zwarte_Prog2,
    SlotKalkulator_Wydawnictwo_Zwarte_Prog3,
)
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte


def ISlot(original, uczelnia=None):
    if isinstance(original, Patent):
        raise CannotAdapt("Sloty dla patentów nie są liczone")

    if hasattr(original, "typ_kbn") and original.typ_kbn.skrot == "PW":
        raise CannotAdapt("Sloty dla prac wieloośrodkowych nie są liczone.")

    if uczelnia is None:
        uczelnia = Uczelnia.objects.get_default()

    if uczelnia is not None and original.status_korekty_id in uczelnia.ukryte_statusy(
        "sloty"
    ):
        raise CannotAdapt(
            "Sloty nie będą liczone, zgodnie z ustawieniami obiektu Uczelnia dla ukrywanych "
            "statusów korekt. "
        )

    if isinstance(original, Wydawnictwo_Ciagle):
        if original.rok in [2017, 2018]:
            if original.punkty_kbn >= 30:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog1(original)
            elif original.punkty_kbn in [20, 25]:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog2(original)
            elif original.punkty_kbn < 20 and original.punkty_kbn > 0:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog3(original)

        elif original.rok > 2018 and original.rok <= PBN_MAX_ROK:
            if original.punkty_kbn in [200, 140, 100]:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog1(original)
            elif original.punkty_kbn in [70, 40]:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog2(original)
            elif original.punkty_kbn <= 20 and original.punkty_kbn > 0:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog3(original)

        raise CannotAdapt(
            "Punkty KBN rekordu (%s) i rok (%s) nie pozwalają na dopasowanie tego rekordu do jakiejkolwiek grupy"
            % (original.punkty_kbn, original.rok)
        )

    elif isinstance(original, Wydawnictwo_Zwarte):
        if original.rok < PBN_MIN_ROK or original.rok > PBN_MAX_ROK:
            raise CannotAdapt(
                "Rok poza zakresem procedur liczacych (%s). " % original.rok
            )

        poziom_wydawcy = -1

        if original.wydawca_id is not None:
            try:
                wydawca = original.wydawca
                poziom_wydawcy = wydawca.get_tier(original.rok)
            except Wydawca.DoesNotExist:
                pass

        # Referaty zjazdowe

        if original.charakter_formalny.charakter_sloty == const.CHARAKTER_SLOTY_REFERAT:

            if (
                original.punkty_kbn == 15
                and original.zewnetrzna_baza_danych.filter(
                    baza__skrot__iexact="wos"
                ).exists()
            ):
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog3(original)

            if original.punkty_kbn in [200, 140, 100]:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog1(original)

            if original.punkty_kbn in [70, 40]:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog2(original)

            if original.punkty_kbn == 20:
                if poziom_wydawcy == 1:
                    return SlotKalkulator_Wydawnictwo_Ciagle_Prog2(original)

                return SlotKalkulator_Wydawnictwo_Ciagle_Prog3(original)

            if original.punkty_kbn == 50 and poziom_wydawcy == 2:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog1(original)

            if original.punkty_kbn == 5:
                return SlotKalkulator_Wydawnictwo_Ciagle_Prog3(original)

            raise CannotAdapt("Nie dopasowano do żadnego rodzaju referatu.")

        # Koniec referatów zjazdowych

        rozdzial = ksiazka = autorstwo = redakcja = False

        if (
            original.charakter_formalny.charakter_sloty
            == const.CHARAKTER_SLOTY_ROZDZIAL
        ):
            rozdzial = True

        if original.charakter_formalny.charakter_sloty == const.CHARAKTER_SLOTY_KSIAZKA:
            ksiazka = True

        if ksiazka and rozdzial:
            raise NotImplementedError("To sie nie powinno wydarzyc)")

        if ksiazka:
            for elem in Typ_Odpowiedzialnosci.objects.filter(
                pk__in=original.autorzy_set.values_list("typ_odpowiedzialnosci_id")
            ).distinct():
                if elem.typ_ogolny == const.TO_AUTOR:
                    autorstwo = True
                    continue
                if elem.typ_ogolny == const.TO_REDAKTOR:
                    redakcja = True
                    continue

            if autorstwo and redakcja:
                raise CannotAdapt("Rekord ma jednocześnie autorów i redaktorów.")

            if not autorstwo and not redakcja:
                raise CannotAdapt("Rekord nie posiada autorów ani redaktorów.")

        tryb_kalkulacji = None

        if ksiazka and autorstwo:
            tryb_kalkulacji = const.TRYB_KALKULACJI.AUTORSTWO_MONOGRAFII

        elif ksiazka and redakcja:
            tryb_kalkulacji = const.TRYB_KALKULACJI.REDAKCJA_MONOGRAFI

        elif rozdzial and autorstwo:
            tryb_kalkulacji = const.TRYB_KALKULACJI.ROZDZIAL_W_MONOGRAFI

        if poziom_wydawcy == 2:
            if (
                (ksiazka and autorstwo and original.punkty_kbn in [200, 100])
                or (ksiazka and redakcja and original.punkty_kbn in [100, 50])
                or (rozdzial and original.punkty_kbn in [50, 25])
            ):
                return SlotKalkulator_Wydawnictwo_Zwarte_Prog1(
                    original, tryb_kalkulacji
                )

        elif poziom_wydawcy == 1:
            if (
                (ksiazka and autorstwo and original.punkty_kbn in [80, 40, 100])
                or (ksiazka and redakcja and original.punkty_kbn in [20, 10])
                or (rozdzial and original.punkty_kbn in [20, 10])
            ):
                return SlotKalkulator_Wydawnictwo_Zwarte_Prog2(
                    original, tryb_kalkulacji
                )

        else:
            if (
                (ksiazka and autorstwo and original.punkty_kbn in [20, 10])
                or (ksiazka and redakcja and original.punkty_kbn in [5, 2.5])
                or (rozdzial and original.punkty_kbn in [5, 2.5])
            ):
                return SlotKalkulator_Wydawnictwo_Zwarte_Prog3(
                    original, tryb_kalkulacji
                )

        raise CannotAdapt(
            "Rekordu nie można dopasować do żadnej z grup monografii. Poziom "
            "wydawcy: %(poziom_wydawcy)s, ksiazka: %(ksiazka)s, rozdzial: %(rozdzial)s, "
            "autorstwo: %(autorstwo)s, redakcja: %(redakcja)s, punkty kbn: %(punkty_kbn)s"
            % dict(
                poziom_wydawcy=poziom_wydawcy,
                ksiazka=ksiazka,
                rozdzial=rozdzial,
                autorstwo=autorstwo,
                redakcja=redakcja,
                punkty_kbn=original.punkty_kbn,
            )
        )

    if hasattr(original, "rok") and hasattr(original, "punkty_kbn"):
        raise CannotAdapt(
            "Nie umiem policzyc dla %s rok %s punkty_kbn %s"
            % (original, original.rok, original.punkty_kbn)
        )

    raise CannotAdapt("Nie umiem policzyć dla obiektu: %r" % original)


class IPunktacjaCacher:
    def __init__(self, original, uczelnia=None):
        self.original = original
        self.slot = None
        self.uczelnia = uczelnia

    def canAdapt(self):
        try:
            self.slot = ISlot(self.original, uczelnia=self.uczelnia)
            return True

        except CannotAdapt:
            return False

    @cached_property
    def ctype(self):
        return ContentType.objects.get_for_model(self.original).pk

    @property
    def cache_punktacja_autora(self):
        return Cache_Punktacja_Autora.objects.filter(
            rekord_id=[self.ctype, self.original.pk]
        )

    @property
    def cache_punktacja_dyscypliny(self):
        return Cache_Punktacja_Dyscypliny.objects.filter(
            rekord_id=[self.ctype, self.original.pk]
        )

    @transaction.atomic
    def removeEntries(self):
        self.cache_punktacja_dyscypliny.delete()
        self.cache_punktacja_autora.delete()

    def serialize(self):
        """
        Zwraca słownik JSON z zawartością danych rekordu
        """
        ret1 = []
        for elem in self.cache_punktacja_autora:
            ret1.append(elem.serialize())

        ret2 = []
        for elem in self.cache_punktacja_dyscypliny:
            ret2.append(elem.serialize())

        return ret1, ret2

    @transaction.atomic
    def rebuildEntries(self):

        pk = (self.ctype, self.original.pk)

        # Jeżeli nie można zaadaptować danego rekordu do kalkulatora
        # punktacji, to po skasowaniu ewentualnej scache'owanej punktacji
        # wyjdź z funkcji:
        if self.canAdapt() is False:
            return

        _slot_cache = {}

        for dyscyplina in self.slot.dyscypliny:
            _slot_cache[dyscyplina] = self.slot.slot_dla_dyscypliny(dyscyplina)
            azd = self.slot.autorzy_z_dyscypliny(dyscyplina)
            if not azd:
                # Na ten moment nie chcemy wpisów odnosnie dyscyplin i slotów, gdy nie ma
                # w nich żadnych autorów (zgłoszenie w Mantis #1009)
                continue

            Cache_Punktacja_Dyscypliny.objects.create(
                rekord_id=[pk[0], pk[1]],
                dyscyplina=dyscyplina,
                pkd=self.slot.punkty_pkd(dyscyplina),
                slot=_slot_cache[dyscyplina],
                autorzy_z_dyscypliny=[a.pk for a in azd],
                zapisani_autorzy_z_dyscypliny=[a.zapisany_jako for a in azd],
            )

        for wa in self.original.autorzy_set.all():
            if (
                not wa.afiliuje
                or not wa.jednostka.skupia_pracownikow
                or not wa.przypieta
            ):
                # Jeżeli autor NIE afiliuje lub afiliuje, ale jednostka nie skupia
                # pracowników (czyli jest OBCA, czyli realnie wpis błędny), lub
                # dyscyplina jest "odpięta" to nie licz punktów dla takiego autora
                continue

            dyscyplina = wa.okresl_dyscypline()
            if dyscyplina is None:
                continue

            pkdaut = self.slot.pkd_dla_autora(wa)
            if pkdaut is None:
                continue

            Cache_Punktacja_Autora.objects.create(
                rekord_id=[pk[0], pk[1]],
                autor_id=wa.autor_id,
                jednostka_id=wa.jednostka_id,
                dyscyplina_id=dyscyplina.pk,
                pkdaut=pkdaut,
                slot=self.slot.slot_dla_autora_z_dyscypliny(dyscyplina),
            )



# Generated by Django 2.1.10 on 2019-10-13 21:24

import bpp.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0181_cache_punktacja_autora_sum_cache_punktacja_autora_sum_gruop'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='uczelnia',
            name='pokazuj_raport_slotow',
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_raport_slotow_autor',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='logged-in', max_length=50, verbose_name='Pokazuj raport slotów - autor'),
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_raport_slotow_uczelnia',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='logged-in', max_length=50, verbose_name='Pokazuj raport slotów - uczelnia'),
        ),
    ]



# Create your models here.

from django.core.exceptions import ValidationError
from django.core.validators import MaxValueValidator
from django.db import models
from django.db.models import JSONField
from django.urls import reverse

from import_common.core import matchuj_autora, matchuj_dyscypline
from . import const
from .fields import LiczbaNField
from .util import InputXLSX, float_or_string_or_int_or_none_to_decimal
from .validators import xlsx_header_validator

from bpp.models import Cache_Punktacja_Autora_Query
from bpp.models.autor import Autor
from bpp.models.dyscyplina_naukowa import Dyscyplina_Naukowa
from bpp.models.uczelnia import Uczelnia


def dyscypliny_naukowe_w_bazie():
    dyscypliny_z_liczba_n = LiczbaNDlaUczelni.objects.values_list(
        "dyscyplina_naukowa", flat=True
    )

    return {
        "pk__in": [
            dyscyplina
            for dyscyplina in Cache_Punktacja_Autora_Query.objects.values_list(
                "dyscyplina", flat=True
            ).distinct()
            if dyscyplina in dyscypliny_z_liczba_n
        ]
    }


class ZamowienieNaRaport(models.Model):
    rodzaj = models.CharField(
        verbose_name="Rodzaj algorytmu",
        max_length=25,
        choices=[
            ("plecakowy", "plecakowy"),
            ("plecakowy_bez_limitu", "plecakowy bez limitu na uczelnię"),
            ("genetyczny", "genetyczny"),
            ("genetyczny_z_odpinaniem", "genetyczny z odpinaniem"),
        ],
    )
    dyscyplina_naukowa = models.ForeignKey(
        Dyscyplina_Naukowa,
        on_delete=models.CASCADE,
        limit_choices_to=dyscypliny_naukowe_w_bazie,
    )
    uid_zadania = models.TextField(blank=True, null=True)
    plik_wyjsciowy = models.FileField()
    wykres_wyjsciowy = models.ImageField()

    ostatnio_zmodyfikowany = models.DateTimeField(auto_now=True)
    status = models.CharField(max_length=255, default="", blank=True, null=True)

    def get_absolute_url(self):
        return reverse("ewaluacja2021:szczegoly-raportu3n", args=(self.pk,))

    class Meta:
        ordering = ("-ostatnio_zmodyfikowany",)


class LiczbaNDlaUczelni(models.Model):
    uczelnia = models.ForeignKey(Uczelnia, on_delete=models.CASCADE)
    dyscyplina_naukowa = models.ForeignKey(Dyscyplina_Naukowa, on_delete=models.CASCADE)
    liczba_n = LiczbaNField()

    class Meta:
        verbose_name = "Liczba N dla uczelni"
        verbose_name_plural = "Liczby N dla uczelni"
        unique_together = [
            ("uczelnia", "dyscyplina_naukowa"),
        ]


class IloscUdzialowDlaAutora(models.Model):
    autor = models.ForeignKey(Autor, on_delete=models.CASCADE)
    dyscyplina_naukowa = models.ForeignKey(Dyscyplina_Naukowa, on_delete=models.CASCADE)
    ilosc_udzialow = LiczbaNField(validators=[MaxValueValidator(4)])
    ilosc_udzialow_monografie = LiczbaNField()

    class Meta:
        verbose_name = "ilość udziałów dla autora"
        verbose_name_plural = "ilości udziałów dla autorów"
        unique_together = [
            (
                "autor",
                "dyscyplina_naukowa",
            )
        ]

    def clean(self):
        if (
            self.ilosc_udzialow is not None
            and self.ilosc_udzialow_monografie is not None
            and self.ilosc_udzialow_monografie > self.ilosc_udzialow
        ):
            raise ValidationError(
                "Ilość udziałów za monografie nie może przekraczać ilości udziałów"
            )


class ImportMaksymalnychSlotow(models.Model):
    header_columns = const.IMPORT_MAKSYMALNYCH_SLOTOW_COLUMNS

    plik = models.FileField(
        validators=[
            xlsx_header_validator(header_columns),
        ]
    )

    started_on = models.DateTimeField(null=True, blank=True)
    ostatnia_zmiana = models.DateTimeField(auto_now=True)
    przeanalizowany = models.BooleanField(default=False)

    class Meta:
        ordering = ("-ostatnia_zmiana",)

    def get_absolute_url(self):
        return reverse("ewaluacja2021:szczegoly-importu", args=(self.pk,))

    def analizuj(self):
        self.wierszimportumaksymalnychslotow_set.all().delete()
        i = InputXLSX(self.plik.path, self.header_columns)
        for elem in i.rows_as_dict():

            wiersz = WierszImportuMaksymalnychSlotow(
                parent=self,
                orig_data=elem,
                matched_autor=matchuj_autora(
                    imiona=elem.get("imie"),
                    nazwisko=elem.get("nazwisko"),
                    orcid=elem.get("orcid"),
                    tytul_str=elem.get("stopien_tytul"),
                ),
                matched_dyscyplina=matchuj_dyscypline(
                    kod=None, nazwa=elem.get("dyscyplina")
                ),
            )

            wiersz.poprawny = wiersz.check_if_valid()
            if wiersz.poprawny:
                wiersz.wymagana_integracja = wiersz.check_if_integration_needed()
            wiersz.save()

        self.przeanalizowany = True
        self.save()

    def integruj(self):
        for wiersz in self.wiersze_do_integracji():
            wiersz.integruj()

    def task_perform(self):
        self.analizuj()
        self.integruj()

    def bledne_wiersze(self):
        return self.wierszimportumaksymalnychslotow_set.exclude(
            poprawny=True
        ).select_related("matched_autor", "matched_dyscyplina", "matched_autor__tytul")

    def wiersze_do_integracji(self):
        return self.wierszimportumaksymalnychslotow_set.filter(
            poprawny=True, wymagana_integracja=True
        ).exclude(zintegrowany=True)

    def wiersze_zintegrowane(self):
        return (
            self.wierszimportumaksymalnychslotow_set.filter(
                poprawny=True, wymagana_integracja=True, zintegrowany=True
            )
            .order_by("matched_autor__nazwisko", "matched_autor__imiona")
            .select_related(
                "matched_autor", "matched_dyscyplina", "matched_autor__tytul"
            )
        )

    def wiersze_nie_zintegrowane(self):
        return (
            self.wierszimportumaksymalnychslotow_set.filter(poprawny=True)
            .exclude(wymagana_integracja=True)
            .order_by("matched_autor__nazwisko", "matched_autor__imiona")
            .select_related(
                "matched_autor", "matched_dyscyplina", "matched_autor__tytul"
            )
        )


class WierszImportuMaksymalnychSlotow(models.Model):
    parent = models.ForeignKey(ImportMaksymalnychSlotow, on_delete=models.CASCADE)

    orig_data = JSONField()

    matched_autor = models.ForeignKey(
        "bpp.Autor", on_delete=models.SET_NULL, null=True, blank=True
    )
    matched_dyscyplina = models.ForeignKey(
        "bpp.Dyscyplina_Naukowa",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )

    poprawny = models.BooleanField(null=True, blank=True, default=None)
    wymagana_integracja = models.BooleanField(null=True, blank=True, default=None)
    zintegrowany = models.BooleanField(default=False)

    @property
    def ilosc_udzialow(self):
        i = self.orig_data.get(
            "maksymalna_suma_udzialow_jednostkowych_wszystkie_dyscypliny"
        )
        return float_or_string_or_int_or_none_to_decimal(i)

    @property
    def ilosc_udzialow_monografie(self):
        i = self.orig_data.get("maksymalna_suma_udzialow_jednostkowych_monografie")
        return float_or_string_or_int_or_none_to_decimal(i)

    def check_if_valid(self):
        ma_autora = self.matched_autor_id is not None
        ma_dyscypline = self.matched_dyscyplina_id is not None
        ma_udzialy = self.ilosc_udzialow is not None and self.ilosc_udzialow != 0
        ma_udzialy_mono = (
            self.ilosc_udzialow_monografie is not None
            and self.ilosc_udzialow_monografie != 0
        )
        if ma_autora and ma_dyscypline and ma_udzialy and ma_udzialy_mono:
            return True

        return False

    def istniejace_dane(self):
        return IloscUdzialowDlaAutora.objects.filter(
            autor=self.matched_autor,
            dyscyplina_naukowa=self.matched_dyscyplina,
        )

    def check_if_integration_needed(self):
        istniejace_dane = self.istniejace_dane().first()

        if istniejace_dane is not None:
            if (
                istniejace_dane.ilosc_udzialow == self.ilosc_udzialow
                and istniejace_dane.ilosc_udzialow_monografie
                == self.ilosc_udzialow_monografie
            ):
                # Wszystko już jest w bazie, integracja zbędna
                return False

        # Brak rekordu lub dane rózne
        return True

    def integruj(self):
        if not self.check_if_integration_needed():
            return

        if self.istniejace_dane().exists():
            i = self.istniejace_dane().first()
            i.ilosc_udzialow = self.ilosc_udzialow
            i.ilosc_udzialow_monografie = self.ilosc_udzialow_monografie
            i.save()
        else:
            IloscUdzialowDlaAutora.objects.create(
                autor=self.matched_autor,
                dyscyplina_naukowa=self.matched_dyscyplina,
                ilosc_udzialow=self.ilosc_udzialow,
                ilosc_udzialow_monografie=self.ilosc_udzialow_monografie,
            )

        self.zintegrowany = True
        self.save()



import pytest
from django.core.exceptions import ValidationError

from bpp.models import Autor_Dyscyplina, Autor_Jednostka
from raport_slotow.models.uczelnia import (
    RaportSlotowUczelnia,
    RaportSlotowUczelniaWiersz,
)


def test_RaportSlotowUczelnia_get_absolute_url(raport_slotow_uczelnia):
    assert raport_slotow_uczelnia.get_absolute_url()


def test_RaportSlotowUczelnia_clean():
    r = RaportSlotowUczelnia(od_roku=2020, do_roku=2010)
    with pytest.raises(ValidationError):
        r.clean()


def test_RaportSlotowUczelnia_create_report(rekord_slotu, rok, raport_slotow_uczelnia):
    raport_slotow_uczelnia.od_roku = rok
    raport_slotow_uczelnia.do_roku = rok
    raport_slotow_uczelnia.save()

    assert RaportSlotowUczelniaWiersz.objects.count() == 0
    raport_slotow_uczelnia.create_report()

    assert RaportSlotowUczelniaWiersz.objects.count() == 1


def test_RaportSlotowUczelnia_create_report_wszystkie(
    rekord_slotu, rok, raport_slotow_uczelnia
):
    raport_slotow_uczelnia.od_roku = rok
    raport_slotow_uczelnia.do_roku = rok
    raport_slotow_uczelnia.akcja = RaportSlotowUczelnia.Akcje.WSZYSTKO
    raport_slotow_uczelnia.save()

    assert RaportSlotowUczelniaWiersz.objects.count() == 0
    raport_slotow_uczelnia.create_report()

    assert RaportSlotowUczelniaWiersz.objects.count() == 1


@pytest.mark.parametrize(
    "pokazuj_zerowych, expected_rows",
    [
        (False, 1),
        (True, 2),
    ],
)
def test_RaportSlotowUczelnia_zerowi_autorzy(
    rekord_slotu,
    rok,
    autor_jan_nowak,
    autor_jan_kowalski,
    raport_slotow_uczelnia,
    pokazuj_zerowych,
    dyscyplina1,
    expected_rows,
    jednostka,
):
    raport_slotow_uczelnia.od_roku = rok
    raport_slotow_uczelnia.do_roku = rok
    raport_slotow_uczelnia.pokazuj_zerowych = pokazuj_zerowych
    raport_slotow_uczelnia.save()

    # Autor "nie-zerowy", przypisanie do dyscypliny
    Autor_Dyscyplina.objects.create(
        rok=rok, autor=autor_jan_kowalski, dyscyplina_naukowa=dyscyplina1
    )
    Autor_Jednostka.objects.create(autor=autor_jan_kowalski, jednostka=jednostka)

    # Autor "zerowy", przypisanie do dyscypliny
    Autor_Dyscyplina.objects.create(
        rok=rok, autor=autor_jan_nowak, dyscyplina_naukowa=dyscyplina1
    )
    Autor_Jednostka.objects.create(autor=autor_jan_nowak, jednostka=jednostka)

    assert RaportSlotowUczelniaWiersz.objects.count() == 0
    raport_slotow_uczelnia.create_report()

    assert RaportSlotowUczelniaWiersz.objects.count() == expected_rows


def test_RaportSlotowUczelnia_autor_niezerowy_jako_zerowy(
    rekord_slotu,
    rok,
    autor_jan_nowak,
    autor_jan_kowalski,
    raport_slotow_uczelnia,
    dyscyplina1,
    dyscyplina2,
):
    raport_slotow_uczelnia.od_roku = rok
    raport_slotow_uczelnia.do_roku = rok
    raport_slotow_uczelnia.pokazuj_zerowych = True
    raport_slotow_uczelnia.save()

    # Autor "nie-zerowy", przypisanie do dyscypliny - i ma on prace w tej dyscypline (rekord_slotu)
    # do tego przypisanie do sub-dyscypliny w ktorej nie ma prac (zeby był jako zerowy rowniez)
    Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
        subdyscyplina_naukowa=dyscyplina2,
    )

    # Autor "zerowy", przypisanie do dyscypliny
    Autor_Dyscyplina.objects.create(
        rok=rok, autor=autor_jan_nowak, dyscyplina_naukowa=dyscyplina1
    )

    assert RaportSlotowUczelniaWiersz.objects.count() == 0
    raport_slotow_uczelnia.create_report()

    assert RaportSlotowUczelniaWiersz.objects.count() == 2


def test_RaportSlotowUczelnia_autor_zerowy_w_jednym_roku_niezerowy_w_innym(
    rekord_slotu,
    rok,
    autor_jan_kowalski,
    raport_slotow_uczelnia,
    dyscyplina1,
):
    raport_slotow_uczelnia.od_roku = rok
    raport_slotow_uczelnia.do_roku = rok + 1
    raport_slotow_uczelnia.pokazuj_zerowych = True
    raport_slotow_uczelnia.save()

    Autor_Dyscyplina.objects.create(
        rok=rok,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
    )
    Autor_Dyscyplina.objects.create(
        rok=rok + 1,
        autor=autor_jan_kowalski,
        dyscyplina_naukowa=dyscyplina1,
    )

    assert RaportSlotowUczelniaWiersz.objects.count() == 0
    raport_slotow_uczelnia.create_report()

    assert RaportSlotowUczelniaWiersz.objects.count() == 1



# Generated by Django 3.0.14 on 2021-10-03 20:39

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0301_doktorska_habilitacyjna_denorm"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql(
                "0302_denorm_autor_dyscyplina_new_trigger"
            )
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0026_auto_20150902_2102'),
    ]

    operations = [
        migrations.AlterField(
            model_name='praca_doktorska',
            name='e_isbn',
            field=models.CharField(db_index=True, max_length=64, null=True, verbose_name=b'E-ISBN', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='isbn',
            field=models.CharField(db_index=True, max_length=64, null=True, verbose_name=b'ISBN', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='e_isbn',
            field=models.CharField(db_index=True, max_length=64, null=True, verbose_name=b'E-ISBN', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='isbn',
            field=models.CharField(db_index=True, max_length=64, null=True, verbose_name=b'ISBN', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='e_isbn',
            field=models.CharField(db_index=True, max_length=64, null=True, verbose_name=b'E-ISBN', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='isbn',
            field=models.CharField(db_index=True, max_length=64, null=True, verbose_name=b'ISBN', blank=True),
            preserve_default=True,
        ),
    ]



# Generated by Django 3.0.9 on 2020-11-11 12:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0223_auto_20200812_1930"),
    ]

    operations = [
        migrations.AddField(
            model_name="autor",
            name="pseudonim",
            field=models.CharField(
                blank=True,
                help_text="\n    Jeżeli w bazie danych znajdują się autorzy o zbliżonych imionach, nazwiskach i tytułach naukowych,\n    skorzystaj z tego pola aby ułatwić ich rozróżnienie. Pseudonim pokaże się w polach wyszukiwania\n    oraz na podstronie autora, po nazwisku, drobnym drukiem. ",
                max_length=300,
                null=True,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-12 22:07
from __future__ import unicode_literals

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0148_charakter_formalny_nadrzedny'),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0149_ranking_afiliacje")
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-12 23:31
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0080_auto_20170708_2147'),
    ]

    operations = [
        migrations.CreateModel(
            name='Konferencja',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ostatnio_zmieniony', models.DateTimeField(auto_now=True, db_index=True, null=True)),
                ('adnotacje', models.TextField(blank=True, db_index=True, help_text='Pole do użytku wewnętrznego -\n    wpisane tu informacje nie są wyświetlane na stronach WWW dostępnych\n    dla użytkowników końcowych.', null=True)),
                ('nazwa', models.TextField(db_index=True, max_length=512)),
                ('skrocona_nazwa', models.CharField(blank=True, max_length=250, null=True, verbose_name='Skrócona nazwa')),
                ('rozpoczecie', models.DateField(verbose_name='Rozpoczęcie')),
                ('zakonczenie', models.DateField(verbose_name='Zakończenie')),
                ('miasto', models.CharField(max_length=100, verbose_name='Miasto')),
                ('panstwo', models.CharField(max_length=100, verbose_name='Państwo')),
                ('baza_scopus', models.BooleanField(default=False, verbose_name='Indeksowana w SCOPUS?')),
                ('baza_wos', models.BooleanField(default=False, verbose_name='Indeksowana w WOS?')),
                ('baza_inna', models.CharField(blank=True, help_text='Wpisz nazwę innej bazy w której indeksowana jest ta konferencja', max_length=200, null=True, verbose_name='Indeksowana w...')),
            ],
            options={
                'verbose_name': 'konferencja',
                'verbose_name_plural': 'konferencje',
                'ordering': ('-rozpoczecie', 'nazwa'),
            },
        ),
        migrations.AlterUniqueTogether(
            name='konferencja',
            unique_together=set([('nazwa', 'rozpoczecie')]),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='konferencja',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='bpp.Konferencja'),
        ),
    ]



# Generated by Django 3.0.14 on 2021-10-04 08:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0302_denorm_autor_dyscyplina_new_trigger"),
    ]

    operations = [
        migrations.DeleteModel(
            name="CacheQueue",
        ),
        migrations.RunSQL("DROP FUNCTION IF EXISTS bpp_wydawca_change CASCADE"),
        migrations.RunSQL(
            "DROP FUNCTION IF EXISTS bpp_alias_wydawcy_change_trigger CASCADE"
        ),
        migrations.RunSQL(
            "DROP FUNCTION IF EXISTS bpp_poziom_wydawcy_change_trigger CASCADE"
        ),
    ]



# Generated by Django 3.0.14 on 2021-05-03 22:11

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0266_auto_20210504_0040"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0265_rekord_mat_doi", app_name="bpp"),
        ),
        # Odbudowanie tabeli rekord_mat spowoduje skasowanie bpp_uczelnia_ewaluacja_view,
        # stąd potrzeba zaciągnąć ten plik z poprzednich migracji:
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0207_uczelnia_analiza_view")
        ),
    ]



# Proste tabele
from dal import autocomplete
from django import forms
from djangoql.admin import DjangoQLSearchMixin

from pbn_api.const import ACTIVE
from pbn_api.models import Journal
from ..models import (  # Publikacja_Habilitacyjna
    Autor,
    Dyscyplina_Zrodla,
    Punktacja_Zrodla,
    Zrodlo,
)
from .core import BaseBppAdminMixin
from .filters import PBN_UID_IDObecnyFilter
from .helpers import (
    ADNOTACJE_FIELDSET,
    CHARMAP_SINGLE_LINE,
    MODEL_PUNKTOWANY_KOMISJA_CENTRALNA,
    MODEL_PUNKTOWANY_Z_KWARTYLAMI_BAZA,
    ZapiszZAdnotacjaMixin,
)

from django.contrib import admin

from bpp.models.zrodlo import Redakcja_Zrodla

# Źródła indeksowane


class Punktacja_ZrodlaForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for field in self.fields:
            self.fields[field].help_text = ""


class Punktacja_ZrodlaInline(admin.TabularInline):
    model = Punktacja_Zrodla
    form = Punktacja_ZrodlaForm
    fields = (
        ("rok",)
        + MODEL_PUNKTOWANY_Z_KWARTYLAMI_BAZA
        + MODEL_PUNKTOWANY_KOMISJA_CENTRALNA
    )
    extra = 1


class Redakcja_ZrodlaForm(forms.ModelForm):
    redaktor = forms.ModelChoiceField(
        queryset=Autor.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:autor-autocomplete"),
    )

    model = Redakcja_Zrodla


class Redakcja_ZrodlaInline(admin.TabularInline):
    model = Redakcja_Zrodla
    extra = 1
    form = Redakcja_ZrodlaForm

    class Meta:
        fields = "__all__"


class Dyscyplina_ZrodlaInline(admin.TabularInline):
    model = Dyscyplina_Zrodla
    extra = 1

    class Meta:
        fields = [
            "dyscyplina",
        ]


class ZrodloForm(forms.ModelForm):

    pbn_uid = forms.ModelChoiceField(
        label="Odpowiednik w PBN",
        required=False,
        queryset=Journal.objects.filter(status=ACTIVE),
        widget=autocomplete.ModelSelect2(
            url="bpp:journal-autocomplete", attrs=dict(style="width: 746px;")
        ),
    )

    class Meta:
        model = Zrodlo
        widgets = {
            "nazwa": CHARMAP_SINGLE_LINE,
            "skrot": CHARMAP_SINGLE_LINE,
            "nazwa_alternatywna": CHARMAP_SINGLE_LINE,
            "skrot_nazwy_alternatywnej": CHARMAP_SINGLE_LINE,
            "poprzednia_nazwa": CHARMAP_SINGLE_LINE,
        }
        fields = "__all__"


class ZrodloAdmin(
    DjangoQLSearchMixin, ZapiszZAdnotacjaMixin, BaseBppAdminMixin, admin.ModelAdmin
):
    djangoql_completion_enabled_by_default = False
    djangoql_completion = True

    form = ZrodloForm

    fields = None
    inlines = (Punktacja_ZrodlaInline, Redakcja_ZrodlaInline, Dyscyplina_ZrodlaInline)
    search_fields = [
        "nazwa",
        "skrot",
        "nazwa_alternatywna",
        "skrot_nazwy_alternatywnej",
        "issn",
        "e_issn",
        "www",
        "poprzednia_nazwa",
        "doi",
        "pbn_uid__pk",
    ]
    autocomplete_fields = ["pbn_uid"]
    list_display = ["nazwa", "skrot", "rodzaj", "www", "issn", "e_issn", "pbn_uid_id"]
    list_filter = [
        "rodzaj",
        "zasieg",
        "openaccess_tryb_dostepu",
        "openaccess_licencja",
        PBN_UID_IDObecnyFilter,
    ]
    list_select_related = ["openaccess_licencja", "rodzaj"]
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "nazwa",
                    "skrot",
                    "rodzaj",
                    "nazwa_alternatywna",
                    "skrot_nazwy_alternatywnej",
                    "issn",
                    "e_issn",
                    "www",
                    "doi",
                    "pbn_uid",
                    "zasieg",
                    "poprzednia_nazwa",
                    "jezyk",
                    "wydawca",
                ),
            },
        ),
        ADNOTACJE_FIELDSET,
        (
            "OpenAccess",
            {
                "classes": ("grp-collapse grp-closed",),
                "fields": (
                    "openaccess_tryb_dostepu",
                    "openaccess_licencja",
                ),
            },
        ),
    )


admin.site.register(Zrodlo, ZrodloAdmin)



from collections import OrderedDict
from enum import Enum

TO_AUTOR = 0
TO_REDAKTOR = 1
TO_INNY = 2
TO_TLUMACZ = 3
TO_KOMENTATOR = 4
TO_RECENZENT = 5
TO_OPRACOWAL = 6
TO_REDAKTOR_TLUMACZENIA = 7

TYP_OGOLNY_DO_PBN = {
    TO_AUTOR: "AUTHOR",
    TO_REDAKTOR: "EDITOR",
    TO_TLUMACZ: "TRANSLATOR",
    TO_REDAKTOR_TLUMACZENIA: "TRANSLATION_EDITOR",
}

GR_WPROWADZANIE_DANYCH = "wprowadzanie danych"
GR_ZGLOSZENIA_PUBLIKACJI = "zgłoszenia publikacji"
GR_RAPORTY_WYSWIETLANIE = "generowanie raportów"
CHARAKTER_SLOTY_KSIAZKA = 1
CHARAKTER_SLOTY_ROZDZIAL = 2
CHARAKTER_SLOTY_REFERAT = 3

RODZAJ_PBN_ARTYKUL = 1
RODZAJ_PBN_ROZDZIAL = 2
RODZAJ_PBN_KSIAZKA = 3
RODZAJ_PBN_POSTEPOWANIE = 4

CHARAKTER_OGOLNY_ARTYKUL = "art"
CHARAKTER_OGOLNY_ROZDZIAL = "roz"
CHARAKTER_OGOLNY_KSIAZKA = "ksi"
CHARAKTER_OGOLNY_INNE = "xxx"


class DZIEDZINA(Enum):
    NAUKI_HUMANISTYCZNE = 1
    NAUKI_INZ_TECH = 2
    NAUKI_MEDYCZNE = 3
    NAUKI_ROLNICZE = 4
    NAUKI_SPOLECZNE = 5
    NAUKI_SCISLE = 6
    NAUKI_TEOLOGICZNE = 7
    NAUKI_SZTUKA = 8


WYZSZA_PUNKTACJA = [
    DZIEDZINA.NAUKI_SPOLECZNE,
    DZIEDZINA.NAUKI_HUMANISTYCZNE,
    DZIEDZINA.NAUKI_TEOLOGICZNE,
]

DZIEDZINY = OrderedDict()
DZIEDZINY[DZIEDZINA.NAUKI_HUMANISTYCZNE] = "Nauki humanistyczne"
DZIEDZINY[DZIEDZINA.NAUKI_INZ_TECH] = "Nauki inżynieryjno-techniczne"
DZIEDZINY[DZIEDZINA.NAUKI_MEDYCZNE] = "Nauki medyczne i o zdrowiu"
DZIEDZINY[DZIEDZINA.NAUKI_ROLNICZE] = "Nauki rolnicze"
DZIEDZINY[DZIEDZINA.NAUKI_SPOLECZNE] = "Nauki społeczne"
DZIEDZINY[DZIEDZINA.NAUKI_SCISLE] = "Nauki ścisłe i przyrodnicze"
DZIEDZINY[DZIEDZINA.NAUKI_TEOLOGICZNE] = "Nauki teologiczne"
DZIEDZINY[DZIEDZINA.NAUKI_SZTUKA] = "Sztuka"


class TRYB_KALKULACJI(Enum):
    AUTORSTWO_MONOGRAFII = 1
    REDAKCJA_MONOGRAFI = 2
    ROZDZIAL_W_MONOGRAFI = 3


class TRYB_DOSTEPU(Enum):
    NIEJAWNY = 0
    TYLKO_W_SIECI = 1
    JAWNY = 2


DO_STYCZNIA_POPRZEDNI_POTEM_OBECNY = "jan_prev_then_current"
NAJWIEKSZY_REKORD = "max_rec"

PBN_UID_LEN = 24
ORCID_LEN = 19

LINK_PBN_DO_AUTORA = "{pbn_api_root}/core/#/person/view/{pbn_uid_id}/current"
LINK_PBN_DO_WYDAWCY = "{pbn_api_root}/core/#/publisher/view/{pbn_uid_id}/current"
LINK_PBN_DO_ZRODLA = "{pbn_api_root}/core/#/journal/view/{pbn_uid_id}/current"
LINK_PBN_DO_PUBLIKACJI = "{pbn_api_root}/core/#/publication/view/{pbn_uid_id}/current"

PBN_LATA = [
    2017,
    2018,
    2019,
    2020,
    2021,
    2022,
    2023,
]

# Minimalny rok od którego zaczynamy liczyć punkty dla prac PBN i w ogóle minimalny rok integracji.
PBN_MIN_ROK = PBN_LATA[0]

# Maksymalny rok dla procedur eksportujących do PBN, liczącyc punkty/sloty oraz testów
PBN_MAX_ROK = PBN_LATA[-1]


KWARTYLE = [(None, "brak"), (1, "Q1"), (2, "Q2"), (3, "Q3"), (4, "Q4")]

WWW_FIELD_LABEL = "Adres WWW (płatny dostęp)"
PUBLIC_WWW_FIELD_LABEL = "Adres WWW (wolny dostęp)"
DOI_FIELD_LABEL = "DOI"
PBN_UID_FIELD_LABEL = "Odpowiednik w PBN"

ZDUBLOWANE_POLE_KOMUNIKAT = (
    'Uwaga, uwaga. W bazie danych istnieją inne rekordy z identycznym polem "{label}". '
    "Technicznie nie jest to błąd, ale mogą pojawić się problemy przy synchronizacji danych "
    "z systemami zewnętrznymi, np z bazą danych PBN."
)

PUSTY_ADRES_EMAIL = "brak@email.pl"



# -*- encoding: utf-8 -*-

from django.test import TestCase

from bpp.models.abstract import ILOSC_ZNAKOW_NA_ARKUSZ
from bpp.models.system import Charakter_Formalny, Jezyk, Typ_KBN
from bpp.tests.tests_legacy.test_reports.util import stworz_obiekty_dla_raportow
from bpp.tests.util import (
    CURRENT_YEAR,
    any_autor,
    any_ciagle,
    any_jednostka,
    any_zwarte,
)
from bpp.views.raporty import get_base_query_jednostka, raport_jednostek_tabela


def _query(model, atrybut, wartosc):
    return model.objects.get(**{atrybut: wartosc})


def charakter(skrot):
    # Pobieramy charaktery w ten sposób, za pomocą funkcji GET, a nie za
    # pomocą zapytań typu charakter_formalny__skrot__in [...] ponieważ
    # pobierając pojedynczo charakter dla każdego skrótu dostaniemy błąd
    # jeżeli dany skrót nie odzwierciedla charakteru w bazie danych
    return _query(Charakter_Formalny, "skrot", skrot)


def typ_kbn(skrot):
    return _query(Typ_KBN, "skrot", skrot)


def jezyk(skrot):
    return _query(Jezyk, "skrot", skrot)


class TestRaportJednostek2012(TestCase):
    # fixtures = ["charakter_formalny.json",
    #             "jezyk.json",
    #             "typ_odpowiedzialnosci.json",
    #             "typ_kbn.json",
    #             "status_korekty.json"]

    def setUp(self):
        stworz_obiekty_dla_raportow()
        Charakter_Formalny.objects.get_or_create(skrot="KS", nazwa="Książka")
        Charakter_Formalny.objects.get_or_create(skrot="ROZ", nazwa="Rozdział książki")

        self.j = any_jednostka()
        self.j.refresh_from_db()

        self.j2 = any_jednostka()
        self.j2.refresh_from_db()

        self.a = any_autor()

    def sprawdz(self, klucz, rezultat, cnt=1, not_found=None):
        base_query = get_base_query_jednostka(self.j, CURRENT_YEAR, CURRENT_YEAR)
        q = raport_jednostek_tabela(klucz, base_query, self.j)
        q = list(q)
        self.assertEqual(len(q), cnt)
        self.assertEqual(q[0].original, rezultat)

        if not_found is not None:
            for elem in not_found:
                self.assertNotIn(elem, q)

    def test_1_1(self):
        c = any_ciagle(
            impact_factor=5,
            punktacja_wewnetrzna=0,
            adnotacje="tei",
            typ_kbn=typ_kbn("PO"),
        )
        c.dodaj_autora(self.a, self.j)

        tego_ma_nie_byc = any_ciagle(
            impact_factor=5,
            punktacja_wewnetrzna=0,
            typ_kbn=Typ_KBN.objects.get(skrot="PW"),
        )
        tego_ma_nie_byc.dodaj_autora(self.a, self.j)

        self.sprawdz("1_1", c, not_found=[tego_ma_nie_byc])

    def test_1_1_zakres_lat(self):
        kwargs = dict(
            impact_factor=5,
            punktacja_wewnetrzna=0,
            adnotacje="foo",
            typ_kbn=typ_kbn("PO"),
        )

        c = any_ciagle(**kwargs)
        d = any_ciagle(rok=CURRENT_YEAR + 5, **kwargs)
        e = any_ciagle(rok=CURRENT_YEAR + 6, **kwargs)
        for elem in [c, d, e]:
            elem.dodaj_autora(self.a, self.j)
        base_query = get_base_query_jednostka(self.j, CURRENT_YEAR, CURRENT_YEAR + 5)
        q = raport_jednostek_tabela("1_1", base_query, self.j).order_by("rok")
        self.assertEqual(q.count(), 2)
        self.assertEqual([x.original for x in list(q)], [c, d])

    def test_1_2(self):
        # Ta praca ma WEJŚC
        common = dict(
            charakter_formalny=charakter("AC"),
            jezyk=jezyk("pol."),
            impact_factor=0,
            punkty_kbn=5,
            liczba_znakow_wydawniczych=500,
        )
        c = any_ciagle(adnotacje="", typ_kbn=typ_kbn("PO"), **common)

        tego_ma_nie_byc = any_ciagle(typ_kbn=Typ_KBN.objects.get(skrot="PW"), **common)
        tego_tez_ma_nie_byc = any_ciagle(adnotacje="wos", **common)
        i_jeszcze_tego_tez = any_ciagle(adnotacje="erih", **common)

        for elem in [c, tego_ma_nie_byc, tego_tez_ma_nie_byc, i_jeszcze_tego_tez]:
            elem.dodaj_autora(self.a, self.j)

        self.sprawdz("1_2", c)

    def test_1_3(self):
        c = any_ciagle(adnotacje="erih", punkty_kbn=1)
        c.dodaj_autora(self.a, self.j)

        tego_ma_nie_byc = any_ciagle(uwagi="erih", punkty_kbn=1)
        tego_ma_nie_byc.dodaj_autora(self.a, self.j)

        self.sprawdz("1_3", c)

    def test_1_4(self):
        c = any_ciagle(
            charakter_formalny=Charakter_Formalny.objects.get(skrot="AC"),
            impact_factor=0,
            punkty_kbn=5,
            liczba_znakow_wydawniczych=1 + (ILOSC_ZNAKOW_NA_ARKUSZ / 2),
            jezyk=Jezyk.objects.get(skrot="ang."),
        )
        c.dodaj_autora(self.a, self.j)
        self.sprawdz("1_4", c)

    def test_1_5(self):
        c = any_ciagle(adnotacje="WOS", punkty_kbn=1)
        c.dodaj_autora(self.a, self.j)

        self.sprawdz("1_5", c)

    def test_2_1(self):
        c = any_ciagle(
            charakter_formalny=charakter("KSZ"), jezyk=jezyk("ang."), punkty_kbn=5
        )
        c.dodaj_autora(self.a, self.j)

        d = any_ciagle(
            charakter_formalny=charakter("KSZ"), jezyk=jezyk("ang."), punkty_kbn=5
        )

        d.dodaj_autora(self.a, self.j2)

        self.sprawdz("2_1", c)

    def test_2_2(self):
        c = any_zwarte(charakter_formalny=charakter("ROZ"), punkty_kbn=20)
        c.dodaj_autora(self.a, self.j)
        self.sprawdz("2_2", c)

    def test_2_3(self):
        d = any_zwarte(
            charakter_formalny=charakter("KSZ"), jezyk=jezyk("ang."), punkty_kbn=50
        )
        d.dodaj_autora(self.a, self.j)

        c = any_zwarte(
            charakter_formalny=charakter("KSZ"), jezyk=jezyk("ang."), punkty_kbn=50
        )
        c.dodaj_autora(self.a, self.j, typ_odpowiedzialnosci_skrot="red.")

        self.sprawdz("2_3", c)



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0010_auto_20141126_1825'),
    ]

    operations = [
        migrations.AddField(
            model_name='patent',
            name='dostep_dnia',
            field=models.DateField(null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='praca_doktorska',
            name='dostep_dnia',
            field=models.DateField(null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='dostep_dnia',
            field=models.DateField(null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='dostep_dnia',
            field=models.DateField(null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='dostep_dnia',
            field=models.DateField(null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
    ]



# -*- encoding: utf-8 -*-

from django.db import models
from django.db.models import CASCADE

from bpp.fields import YearField
from bpp.models import Wydawnictwo_Zwarte
from bpp.models.autor import Autor
from bpp.models.struktura import Wydzial
from bpp.util import zrob_cache


class Opi_2012_Afiliacja_Do_Wydzialu(models.Model):
    """Tymczasowa klasa, zawierająca przypisania autorów do wydziałów, ponieważ
    dla pewnej grupy autorów otrzymałem w plikach XLS jedynie afiliację do wydziału,
    nie zaś do jednostki.
    """
    autor = models.ForeignKey(Autor, CASCADE)
    wydzial = models.ForeignKey(Wydzial, CASCADE)
    rok = YearField(db_index=True)

    class Meta:
        unique_together = [('autor', 'wydzial', 'rok'),]
        app_label = 'bpp'


class Opi_2012_Tytul_Cache_Manager(models.Manager):
    def rebuild(self):
        self.all().delete()
        for w in Wydawnictwo_Zwarte.objects.all():
            c = zrob_cache(w.tytul_oryginalny)
            self.create(
                wydawnictwo_zwarte=w,
                tytul_oryginalny_cache=c
            )


class Opi_2012_Tytul_Cache(models.Model):
    """Ta klasa to cache tytułów publikacji - wszystkie litery zmniejszone,
    wycięte znaki przestankowe typu kropka lub przecinek, wycięte spacje.
    """
    wydawnictwo_zwarte = models.ForeignKey(Wydawnictwo_Zwarte, CASCADE)
    tytul_oryginalny_cache = models.TextField(db_index=True)

    objects = Opi_2012_Tytul_Cache_Manager()

    class Meta:
        app_label = 'bpp'



from denorm import denormalized, depend_on_related
from django.db import models
from django.db.models import CASCADE, SET_NULL, JSONField

from django.contrib.postgres.fields import ArrayField

from django.utils.functional import cached_property

from bpp.models import (
    BazaModeluOdpowiedzialnosciAutorow,
    ModelOpcjonalnieNieEksportowanyDoAPI,
    ModelPunktowany,
    ModelRecenzowany,
    ModelZAdnotacjami,
    ModelZeStatusem,
    ModelZeSzczegolami,
    ModelZInformacjaZ,
    ModelZPrzeliczaniemDyscyplin,
    ModelZRokiem,
    ModelZWWW,
)
from bpp.models.abstract import (
    DodajAutoraMixin,
    MaProcentyMixin,
    ModelZAbsolutnymUrl,
    RekordBPPBaza,
)
from bpp.models.autor import Autor
from bpp.models.system import Charakter_Formalny, Jezyk
from bpp.util import safe_html


class Patent_Autor(BazaModeluOdpowiedzialnosciAutorow):
    """Powiązanie autora do patentu."""

    rekord = models.ForeignKey("Patent", CASCADE, related_name="autorzy_set")

    class Meta:
        verbose_name = "powiązanie autora z patentem"
        verbose_name_plural = "powiązania autorów z patentami"
        app_label = "bpp"
        ordering = ("kolejnosc",)
        unique_together = [
            ("rekord", "autor", "typ_odpowiedzialnosci"),
            # Tu musi być autor, inaczej admin nie pozwoli wyedytować
            ("rekord", "autor", "kolejnosc"),
        ]


class _Patent_PropertyCache:
    @cached_property
    def charakter_formalny(self):
        return Charakter_Formalny.objects.get(skrot="PAT")

    @cached_property
    def jezyk(self):
        return Jezyk.objects.get(nazwa__icontains="polski")


_Patent_PropertyCache = _Patent_PropertyCache()


class Patent(
    RekordBPPBaza,
    ModelZRokiem,
    ModelZeStatusem,
    ModelZWWW,
    ModelRecenzowany,
    ModelPunktowany,
    ModelZeSzczegolami,
    ModelZInformacjaZ,
    ModelZAdnotacjami,
    MaProcentyMixin,
    DodajAutoraMixin,
    ModelZAbsolutnymUrl,
    ModelOpcjonalnieNieEksportowanyDoAPI,
    ModelZPrzeliczaniemDyscyplin,
):
    tytul_oryginalny = models.TextField("Tytuł oryginalny", db_index=True)

    data_zgloszenia = models.DateField("Data zgłoszenia", null=True, blank=True)

    numer_zgloszenia = models.CharField(
        "Numer zgłoszenia", max_length=255, null=True, blank=True
    )

    data_decyzji = models.DateField(null=True, blank=True)

    numer_prawa_wylacznego = models.CharField(
        "Numer prawa wyłącznego", max_length=255, null=True, blank=True
    )

    rodzaj_prawa = models.ForeignKey(
        "bpp.Rodzaj_Prawa_Patentowego", CASCADE, null=True, blank=True
    )

    wdrozenie = models.BooleanField("Wdrożenie", null=True, blank=True, default=None)

    wydzial = models.ForeignKey("bpp.Wydzial", SET_NULL, null=True, blank=True)

    autor_rekordu_klass = Patent_Autor
    autorzy = models.ManyToManyField(Autor, through=autor_rekordu_klass)

    class Meta:
        verbose_name = "patent"
        verbose_name_plural = "patenty"
        app_label = "bpp"

    def __str__(self):
        return self.tytul_oryginalny

    @cached_property
    def charakter_formalny(self):
        return _Patent_PropertyCache.charakter_formalny

    @cached_property
    def jezyk(self):
        return _Patent_PropertyCache.jezyk

    def clean(self):
        # DwaTytuly.clean() w wydaniu jedno-tytułowym...
        self.tytul_oryginalny = safe_html(self.tytul_oryginalny)

    #
    # Cache framework by django-denorm-iplweb
    #

    denorm_always_skip = ("ostatnio_zmieniony",)

    @denormalized(JSONField, blank=True, null=True)
    @depend_on_related(
        "bpp.Patent_Autor",
        only=(
            "autor_id",
            "jednostka_id",
            "typ_odpowiedzialnosci_id",
            "afiliuje",
            "dyscyplina_naukowa_id",
            "upowaznienie_pbn",
            "przypieta",
        ),
    )
    def cached_punkty_dyscyplin(self):
        # TODO: idealnie byłoby uzależnić zmiane od pola 'rok' które by było identyczne
        # dla bpp.Poziom_Wydawcy, rok i id z nadrzędnego. Składnia SQLowa ewentualnie
        # jakis zapis django-podobny mile widziany.
        return self.przelicz_punkty_dyscyplin()

    @denormalized(models.TextField, default="")
    @depend_on_related(
        "bpp.Patent_Autor",
        only=("zapisany_jako", "typ_odpowiedzialnosci_id", "kolejnosc"),
    )
    @depend_on_related("bpp.Status_Korekty")
    def opis_bibliograficzny_cache(self):
        return self.opis_bibliograficzny()

    @denormalized(ArrayField, base_field=models.TextField(), blank=True, null=True)
    @depend_on_related(
        "bpp.Autor",
        only=(
            "nazwisko",
            "imiona",
        ),
    )
    @depend_on_related("bpp.Patent_Autor", only=("kolejnosc",))
    def opis_bibliograficzny_autorzy_cache(self):
        return [
            f"{x.autor.nazwisko} {x.autor.imiona}" for x in self.autorzy_dla_opisu()
        ]

    @denormalized(models.TextField, blank=True, null=True)
    @depend_on_related(
        "bpp.Patent_Autor",
        only=("zapisany_jako", "kolejnosc"),
    )
    def opis_bibliograficzny_zapisani_autorzy_cache(self):
        return ", ".join([x.zapisany_jako for x in self.autorzy_dla_opisu()])

    @denormalized(
        models.SlugField,
        max_length=400,
        unique=True,
        db_index=True,
        null=True,
        blank=True,
    )
    @depend_on_related(
        "bpp.Patent_Autor",
        only=("zapisany_jako", "kolejnosc"),
    )
    @depend_on_related(
        "bpp.Autor",
        only=("nazwisko", "imiona"),
    )
    def slug(self):
        return self.get_slug()



# -*- encoding: utf-8 -*-
from django.template import Library

register = Library()


def find(ciag, znaki):
    if ciag is None:
        return

    return ciag.find(znaki) >= 0


register.filter(find)


# -*- encoding: utf-8 -*-
try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from django.test import TestCase
from django.test.client import Client


class Test(TestCase):
    def setUp(self):
        pass

    def test_raport_autorow(self):
        c = Client()
        res = c.get(reverse("bpp:raport_autorow_formularz"))
        self.assertEquals(res.status_code, 200)
